#harmonize stage records by correction with manual recordings and stitching multiple logger files together


#settings
meas_file="qmeas_isab_hom.txt" #table with manual stage measurements (columns "date", "gauge" and  "stage_.m" required)

gauges2do=c("Villacarli") #name of gauge(s) to treat (leave empty for all)
gauges2do=c("Carrasquero") #name of gauge(s) to treat (leave empty for all)
gauges2do=c("Ceguera") #name of gauge(s) to treat (leave empty for all)
gauges2do=c("Cabecera") #name of gauge(s) to treat (leave empty for all)

#todo


gauges2do=c("Lascuarre") #name of gauge(s) to treat (leave empty for all)

stage_file_dir="./hdata/" #directory containing stage records
logger_meta_file="observation_periods.txt" #file containing information on logger runtime. Will be supplemented with correction coefficients
tolerance_time= as.difftime(40, units="mins") #tolerance time for including manual measurements into correction of logger period

residuals_warning = 0.05 #threshold for warning, when logger readings could not be fitted to observations [units of manual readings records]
same_datum_readings_only=FALSE #if set to FALSE, manual readings of other datum origin are converted and used 
prevent_negative_stage = FALSE #if enabled, the intercept is adjusted to prevent negative values ()
do_plot="all" #plot "all" periods, "problematic" ones or "none"
new_win=TRUE #separate window for each plot

skip_existing=FALSE #TRUE: coefficients existing in file are not altered; FALSE: update all coefficients
write_converted_series=TRUE #should the corrected series be written?
aggregate_logger_periods= FALSE #FALSE: aggregate only logger periods by column "logger_period" (i.e. use different coefficients for different subsets)  
                                #TRUE: aggregate all periods of a gauge/datum/device (results in the same calibration coefficients for all periods)
identify_stage_measurements=FALSE #allows interactive identification of stage measurements in plot
compare_seams=FALSE #plot adjacent periods to check if they match
read_comments=FALSE  #allow entering comments for periods

#end settings
graphics.off()

#prepare datum conversion
print("qmeas_isab_hom.txt generated?")
# source("convert_stage.R") #include conversion functions
# convert_info=read.table(file="../wq/gauge_datum_conv.txt", header=TRUE, sep="\t", fill=TRUE)

#read manual stage measurements
meas = read.table(file=meas_file, header=TRUE, sep="\t", stringsAsFactors=FALSE, na.strings=c("NA","?","na"))
names(meas)[grepl(pattern="stage_\\.",names(meas))]="stage"

meas$converted_stage=0*NA #temporary converted stage


if (grepl(meas$date[1], pattern = "-"))
  format="%Y-%m-%d %H:%M" else
  format="%d.%m.%Y %H:%M"
meas$date=as.POSIXct(strptime(meas$date, format=format, tz="etc/GMT-1")) #default: GMT+1
if (all(is.na(meas$date))) stop(paste0("wrong date format in ",meas_file))

unknown_time= (meas$date == round(meas$date, units="day")) | is.na(meas$date)
meas=meas[!unknown_time,] #don't use entries with lacking time stamp (i.e. date only)
meas$weight..0.1.[is.na(meas$weight..0.1.)]=1 #unset weight is interpreted as 1
meas=meas[meas$weight..0.1.!=0,] #discard entries with zero weight
meas$by[is.na(meas$by)]=""



#read logger meta data  
  source("f_read_loggerdata.R")
 
  logger_meta_org = read_logger_meta(logger_meta_file = "observation_periods.txt")
  start_org = logger_meta_org$Start #store original data
  end_org   = logger_meta_org$End
  


find_closest=function(series, target) #find closest match of target in series
{return(which.min(abs(series-target))[1])}



#add columns for logging recalibration reports   
if (is.null(logger_meta_org$no_readings)) logger_meta_org$no_readings=NA #create columns if not already there
if (is.null(logger_meta_org$intercept)) logger_meta_org$intercept=NA 
if (is.null(logger_meta_org$slope)) logger_meta_org$slope=NA 
if (is.null(logger_meta_org$r2)) logger_meta_org$r2=NA 
if (is.null(logger_meta_org$rmse)) logger_meta_org$rmse=NA 

logger_meta_out = logger_meta_org

#logger_data$corrected=FALSE  
if (length(gauges2do)==0)
  gauges2do = unique (meas$gauge)
  
#gauge="Cabecera"; datum="weir"; device="TT"
for (gauge in gauges2do) #loop over all gauges, stage datums and devices
  for (datum in unique (logger_meta_org$stage_datum[logger_meta_org$Gauge==gauge]))
    for (device in unique (logger_meta_org$Device[logger_meta_org$Gauge==gauge & logger_meta_org$stage_datum==datum  ]))
    {
      #if (datum=="bridge") next
      
      already_skipped=NULL
      print(paste(gauge,"/",datum,"/",device))
      
      
      current_meta_recs=logger_meta_org$Gauge==gauge &
        logger_meta_org$stage_datum==datum &
        logger_meta_org$Device     ==device #meta records of current gauge
      current_meta_recs_no = which(current_meta_recs) #convert to IDs (i.e. line numbers)
      
      if (skip_existing & all(!is.na(logger_meta_org$intercept[current_meta_recs]))& all(!is.na(logger_meta_org$slope[current_meta_recs])) & !write_converted_series)
      {
        print(paste("all done for",gauge,"/",datum,"/",device,", skipped."))
        next
      }
      
      logger_data=na.omit(read_loggerdata(gauge, datum, device, stage_file_dir=stage_file_dir, do_correct_tz = TRUE)) #try to read logger data
      
      if (is.null(logger_data))
      {
        print(paste("no data found for",gauge,"/",datum,"/",device,"."))
        next
      }
      
      
      breaks = which(diff(logger_data$date)<0) #find points of time overlap due to daylight saving
      
      #attr(a,"tzone")="etc/GMT-1"
      #logger_data$date[sort(c(breaks, breaks-1, breaks+1))]
      
      if (any(breaks))
      {
        if (length(breaks) > difftime(max(logger_data$date), min(logger_data$date), units = "days")/365)
            warning("(too) many overlaps found in time. Presumably Omnilog export error (check midnight timestamps)")
        break_periods = apply(matrix(sapply(X=1:nrow(logger_data), FUN=get(">"), breaks), nrow=length(breaks)), MAR=2, sum) #number periods between breaks
        
        stop(paste("GMT+2 to GMT+1 breaks found at:", paste(logger_data$date[breaks], collapse=", ")," check observation_periods.txt"))

      }
      
      current_man_reads= meas$gauge==gauge   #relevant manual readings
      same_datum_i = current_man_reads & meas$stage_datum==datum
      if (same_datum_readings_only) 
        current_man_reads=same_datum_i else  #use only readings of the same datum
      {
        double_readings_i = (meas$date %in% meas$date[same_datum_i]) &
          !same_datum_i & current_man_reads  #find manual readings that are already covered by duplicate readings in the current datum
        current_man_reads = current_man_reads & !double_readings_i #ignore them
        
#         meas$converted_stage[current_man_reads] =    #convert all available readings into current datum
#           convert_stage(gauge=gauge, convert_info=convert_info, 
#                         from_datum=meas$stage_datum[current_man_reads],
#                         to_datum=datum,
#                         stage=meas$stage[current_man_reads])
        meas$converted_stage[current_man_reads]=meas$stage[current_man_reads] #no more conversion required since we use alread homogenized file
        current_man_reads = current_man_reads & is.finite(meas$converted_stage)  #ignore nodata
        
      
        #disable duplicates that originated from two source datums to current target
        double_readings_i       = duplicated(meas$date[current_man_reads]) 
#         browser()
#          meas$date[current_man_reads][double_readings_i]
#          a=which(meas$date[current_man_reads]==meas$date[current_man_reads][double_readings_i][4])
#          meas[current_man_reads,][a,]
        
        if (any(double_readings_i))
        {  
          double_readings_i_both  = meas$date[current_man_reads] %in% meas$date[current_man_reads][double_readings_i] #find all duplicates and their originals
          aggr_fun = function(x) ifelse(all(is.numeric(x)), mean(x, na.rm=TRUE), paste(x, collapse="/"))
          aggregated = aggregate(meas[current_man_reads, c("converted_stage", "stage_datum")], by=list(date=meas$date[current_man_reads]), FUN=aggr_fun)
          merged = merge(meas[current_man_reads, c("date","source")], aggregated, by="date")
          meas$converted_stage[current_man_reads][double_readings_i_both]  =  merged$converted_stage[double_readings_i_both]
          meas$stage_datum    [current_man_reads][double_readings_i_both]  =  merged$stage_datum[double_readings_i_both]
          current_man_reads[double_readings_i] = FALSE #disregard the duplicates 
        }
      }
      
      #unknown start or end times are denoted by 0:00; set these to adjacent period
      unknown_start= logger_meta_org$Start[current_meta_recs] == round(logger_meta_org$Start[current_meta_recs], units="day")
      unknown_start[1]=FALSE #first period cannot be corrected
      logger_meta_org$Start[current_meta_recs][unknown_start] = logger_meta_org$End[current_meta_recs][which(unknown_start)-1] + as.difftime(5, units="mins")  
      
      unknown_end= logger_meta_org$End[current_meta_recs] == round(logger_meta_org$End[current_meta_recs], units="day")
      unknown_end[length(unknown_end)]=FALSE #last period cannot be corrected
      logger_meta_org$End[current_meta_recs][unknown_end] = logger_meta_org$Start[current_meta_recs][which(unknown_end)+1] - as.difftime(5, units="mins")  
      
      
      print(paste(": logger periods:", sum(current_meta_recs),
                  "; total readings:", sum(current_man_reads)))
      
      logger_meta_org$no_readings[current_meta_recs]=NA #number of readings per logger period
      #count number of manual reading per logger period
      for (periodi in which(current_meta_recs))
      {
        #find manual readings for current logger period
        manual_readings_i = meas$date[current_man_reads] >= logger_meta_org$Start[periodi]-tolerance_time & 
          meas$date[current_man_reads] <= logger_meta_org$End[periodi]  +tolerance_time
        logger_meta_org$no_readings[periodi] = sum(manual_readings_i)
      }  
      #browser()
      print(paste("number of readings per period (added to file",logger_meta_file,"):"))
      print(logger_meta_org[current_meta_recs,c("Start","End","no_readings")])
      
      
      #check seams between adjacent periods for matching (i.e. getting the same calibration constants)
      if (compare_seams & (sum(current_meta_recs)>1))
      for (period_ix in 2:sum(current_meta_recs))
      {
        if (abs(difftime(logger_meta_org$End  [current_meta_recs][period_ix-1],
                     logger_meta_org$Start[current_meta_recs][period_ix], units="hours")) > 6)
          next #if the gap is larger than 6 hours, skip
        
        #select seam
        selection = logger_data$date >= logger_meta_org$End  [current_meta_recs][period_ix-1] - 24*3600 &
                    logger_data$date <= logger_meta_org$Start[current_meta_recs][period_ix  ] + 24*3600 
        if (!any(selection))
          next
        rge = diff(range(logger_data$stage[selection])) #for rescaling
        if (rge==0) rge=1
        plot(logger_data$date[selection], logger_data$stage[selection], pch="." ,
             main=paste("Periods starting ", paste(logger_meta_org$Start[current_meta_recs][(-1:0)+period_ix  ], collapse=", "),"\n",
                        "logger_periods:", paste(logger_meta_org$logger_period[current_meta_recs][(-1:0)+period_ix  ], collapse=", ")),
        ylim=extendrange(logger_data$stage[selection], f=0.10 / rge
        ))
                abline(v=logger_meta_org$Start[current_meta_recs][period_ix  ])

                  commnt=readline("press any key ")
        
        }
      
   
      current_meta_recs_org = current_meta_recs
      if (aggregate_logger_periods)
        by=as.list(logger_meta_org[, c(                 "stage_datum", "Gauge", "Device")]) else
        by=as.list(logger_meta_org[, c("logger_period", "stage_datum", "Gauge", "Device")])  
      
      agg_fun=function(x, minmax=min)
      {  
        if (class(x)[1]=="POSIXct")
          ret_val = minmax(x) else #return min/max
            ret_val = x[1]  #return first entry
        return(ret_val)
      }  
      
      start_dates=aggregate(x=logger_meta_org[,!names(logger_meta_org) %in% c("End", "no_readings", names(by))], by=by, FUN=agg_fun, minmax=min)
        names(start_dates)[names(start_dates)=="x"]="Start"
      end_dates  =aggregate(x=logger_meta_org[, "End"  ], by=by, FUN=max)
      names(end_dates)  [names(end_dates  )=="x"]="End"
      no_readings  =aggregate(x=logger_meta_org[, "no_readings"  ], by=by, FUN=sum)
        names(no_readings)  [names(no_readings  )=="x"]="no_readings"
      
      remarks_merged = aggregate(logger_meta_org$remarks, by=by, FUN=paste, collapse="")
      remarks_merged =gsub(pattern = "+", replacement = "", remarks_merged$x)  
      start_dates$remarks_merged = remarks_merged
      #browser()
      logger_meta_aggr = merge(merge(start_dates, end_dates), no_readings) #modified logger_meta_aggr with aggregated periods 
      logger_meta_aggr$remarks="" #will be restored later
      logger_meta_aggr$treated=FALSE #flag treated records
      current_meta_recs=logger_meta_aggr$Gauge==gauge &
        logger_meta_aggr$stage_datum==datum &
        logger_meta_aggr$Device     ==device #meta records of current gauge
      current_meta_recs_no = which(current_meta_recs) #convert to IDs
      
      
      #browser()
      #iterative period calibration
      covered_meta_recs_no = which(current_meta_recs & logger_meta_aggr$no_readings>0) #find periods that can be corrected
      covered_meta_recs_no = covered_meta_recs_no[sort.int(logger_meta_aggr$no_readings[covered_meta_recs_no], index.return=TRUE, decreasing=TRUE)$ix] #sort by number of valid measurements
      if (!skip_existing) logger_meta_aggr$slope[current_meta_recs][logger_meta_aggr$calibrated[current_meta_recs]==0] = NA #set non-locked slope values to NA
      
      while (length(covered_meta_recs_no)>0)  #while not all corrected
      {
        periodi = covered_meta_recs_no[1] #treat this period (the one with the most measurements)
        covered_meta_recs_no = covered_meta_recs_no[-1] #remove from list
        period_label=paste(format(logger_meta_aggr[periodi, c("Start", "End")], format="%Y-%m-%d %H:%M", tz="etc/GMT-1"),collapse=" - ")
        
        
        #find potential manual readings for current logger period
        manual_readings_i = meas$date[current_man_reads] >= logger_meta_aggr$Start[periodi]-tolerance_time & 
          meas$date[current_man_reads] <= logger_meta_aggr$End[periodi]  +tolerance_time
        logger_meta_aggr$no_readings[periodi] = sum(manual_readings_i)  #update value (may have been overestimated before due to optimistic increasing because of adjacent periods)
        #browser()
        print(paste(" calibrate period",period_label, "(", logger_meta_aggr$no_readings[periodi]," man. readings)"))
        
        if (logger_meta_aggr$no_readings[periodi] == 0)            #no manual reading, no correction possible
        {
          print(paste(" No manual readings found for period ",period_label, ", skipped."))
          current_meta_recs[periodi]=FALSE #this period cannot be treated
          next
        }
          

        reading_dates = meas$date[current_man_reads][manual_readings_i] #dates of manual readings
        
        whole_period_i = logger_data$date >= logger_meta_aggr$Start[periodi] & logger_data$date <= logger_meta_aggr$End[periodi] #index to whole logging period
        if (!any(whole_period_i))
        {
 #         browser()
          print(paste(" No logger data found for period ",period_label, ", skipped."))
          current_meta_recs[periodi]=FALSE #this period cannot be treated
          next
        }
        
        if (all(is.na(logger_data$stage[whole_period_i])))
        {
          print(paste(" Only NAs found for period ",period_label, ", skipped."))
          current_meta_recs[periodi]=FALSE #this period cannot be treated
          next
        }
        
        logger_data_i = sapply(X=reading_dates, FUN=find_closest, series=logger_data$date[whole_period_i]) #find indices to dates that correspond to times of manual readings
        logger_data_i = which(whole_period_i)[logger_data_i]  #convert to absolute index of logger_data
        
        time_mismatch = abs(difftime(reading_dates, logger_data$date[logger_data_i], units="min" )) #temporal mismatch between manual and closest logger reading
        
        #check for duplicates
          a=table(logger_data_i)
          duplicate_matches = names(a)[a>1] #mark logger readings that are related to more than one manual measurement
          for (m in duplicate_matches)
          {
            ix = which(logger_data_i==m) #select current group of duplicate matches
            seam_meas = meas$method[current_man_reads][manual_readings_i][ix] %in% c("prev_period", "next_period") #which of the duplicate measurements originate from seams to adjacent periods?
            
            closest_meas= which.min(time_mismatch[ix])  #find which of the matches comes closest to time of logger reading
            keep_these =c(closest_meas, which(seam_meas))
            time_mismatch[ix[-keep_these]] = Inf #mark all other measurements as "too far", willbe removed later
          }          
        
        #ignore readings that are too far away (or duplicated)
        manual_readings_i_matched=manual_readings_i 
        too_far = time_mismatch > tolerance_time
        manual_readings_i_matched[which(manual_readings_i)[too_far]] = FALSE
        logger_data_i = logger_data_i [!too_far] 

        if (!any(manual_readings_i_matched))
        {
          if (!(periodi %in% already_skipped))
            {
              already_skipped=c(already_skipped,periodi)
              covered_meta_recs_no = c(covered_meta_recs_no, periodi) #no valid readings left, postpone calibration
          }
          next
        }
    
        if (skip_existing)
        {
          slope    =logger_meta_aggr$slope[periodi]
          if (is.na(slope)) slope=1
          intercept=logger_meta_aggr$intercept[periodi]
          r2 = NA
        }  else
        {  
          #default: simple intercept correction
          slope=logger_meta_aggr$slope[periodi]
          if (is.na(slope)) slope=1
          intercept = mean(meas$converted_stage[current_man_reads][manual_readings_i_matched]) - slope*mean(logger_data$stage[logger_data_i], na.rm=TRUE)
          r2 = 1
          
          if (!logger_meta_aggr$calibrated[periodi]==1 &  #logger period not assigned as "calibrated"
                logger_meta_aggr$no_readings[periodi]>1 &  #more than one measurement available
                sum(meas$by[current_man_reads][manual_readings_i_matched]!="script") >= 2 ) #and at least 2 real measurements in period
              #adjust slope, too        
          {
            recalib_mod = lm(formula=stage~logger_val, weights =  meas$weight..0.1.[current_man_reads][manual_readings_i_matched],
                               data=data.frame(stage=meas$converted_stage[current_man_reads][manual_readings_i_matched], logger_val=logger_data$stage[logger_data_i]))  
            if (!is.na(coef(recalib_mod)[2])) #regression successful (failure may happen if all datapoints are identical)
            {
              slope=coef(recalib_mod)[2]
              intercept = coef(recalib_mod)[1]
              #r2 = cor(meas$converted_stage[current_man_reads][manual_readings_i_matched], logger_data$stage[logger_data_i])^2   
            }
          } 
          
          if (!is.na(slope) && (slope < 0)) 
            print(paste(" Period ",period_label, "results in unusual slope correction of", slope))
        }  
        
        corrected_loggerdata = slope*logger_data$stage[whole_period_i] + intercept
        do_plot2 = (do_plot=="all")
        
        if (any(corrected_loggerdata < 0)) 
        {  
          print(paste(" Period ",period_label, "results in negative stage values."))
          if (do_plot!= "none") do_plot2 = TRUE 
          if(prevent_negative_stage)
          {
            print("intercept adjusted")
            adjustment = min(corrected_loggerdata)
            intercept = intercept - adjustment 
            corrected_loggerdata = corrected_loggerdata -adjustment
          }
        }  
        
        
        resid = meas$converted_stage[current_man_reads][manual_readings_i_matched] -  (logger_data$stage[logger_data_i] * slope + intercept)
        r2 = 1-var(resid)/var(meas$converted_stage[current_man_reads][manual_readings_i_matched])
        
        #plot fit
        if (new_win | identify_stage_measurements) windows()          #open new window, if requested
        plot(logger_data$stage[logger_data_i], meas$converted_stage[current_man_reads][manual_readings_i_matched],
             col = as.factor(meas$stage_datum[current_man_reads])[manual_readings_i_matched],
             pch = 20 +     (meas$by         [current_man_reads][manual_readings_i_matched]=="script"),
             main=paste(periodi,"\n", period_label, "\nslope=", slope,"; intercept=", intercept, sep=""), xlab="logger val", ylab="meas stage")
        lpoints=c(0,max(logger_data$stage[logger_data_i])*5) #for plotting regression line
        lines(lpoints, slope*lpoints+intercept)
        legend("bottom", legend=c(unique(meas$stage_datum[current_man_reads][manual_readings_i_matched]), " reading", " adj.period"),
               pch=c(rep(20, length(unique(meas$stage_datum[current_man_reads][manual_readings_i_matched]))+1), 21), 
               col=c(unique(as.factor(meas$stage_datum[current_man_reads])[manual_readings_i_matched]),1,1))
        
        if (identify_stage_measurements)
          identify(logger_data$stage[logger_data_i], meas$converted_stage[current_man_reads][manual_readings_i_matched], labels = meas$date[current_man_reads][manual_readings_i_matched])

        #store recalibrated data
        logger_data$stage_calibrated[whole_period_i] = corrected_loggerdata
        
                
        if (any(abs(resid) > residuals_warning)) 
        {
          print(paste(" Period ", period_label, "results in large residuals of", paste(resid, collapse=",") ))
          if (do_plot!= "none") do_plot2 = TRUE 
        }
        
        if (do_plot2)
        {  
          #if (main_win!=0)
          #  dev.set(main_win)
          if (new_win) windows()          #open new window, if requested
          
          plot(logger_data$date[whole_period_i],logger_data$stage_calibrated[whole_period_i], type="l",
               xlim=extendrange(logger_data$date[whole_period_i]), 
               ylim=range(c(meas$converted_stage[current_man_reads][manual_readings_i], logger_data$stage_calibrated[whole_period_i]), na.rm=TRUE),
               xlab=period_label)
          
          points(meas$date[current_man_reads][manual_readings_i],meas$converted_stage[current_man_reads][manual_readings_i], 
                 col= as.factor(meas$stage_datum[current_man_reads])[manual_readings_i],
                 pch = 20 +     (meas$by         [current_man_reads][manual_readings_i]=="script"))
          
       
          
          
          
                    #main_win=dev.cur()
          ttt= logger_meta_org$Gauge==gauge & logger_meta_org$Device==device & logger_meta_org$stage_datum==datum 
          abline(v=logger_meta_org$Start[ttt], col="green", lty="dashed") #plot start and end of original periods
          abline(v=logger_meta_org$End  [ttt], col="red", lty="dotted")
          
          
          
          print(paste("current comment(s): ",logger_meta_aggr$remarks_merged  [periodi] ))
          #logger_meta_org$remarks[logger_meta_org$Start >= logger_meta_aggr$Start  [periodi] && logger_meta_org$End >= logger_meta_aggr$End  [periodi] ]
          if (read_comments)
          {
            commnt=readline("type ENTER to continue, new comment, 'd' to debug or 'q' to break ")
            if (commnt=="q") break else
              if (commnt=="d") browser() else
                if (commnt!="") logger_meta_aggr$remarks  [periodi] =  commnt
          }
        }
        
        #log recalibration specs
        logger_meta_aggr$intercept  [periodi] = intercept
        logger_meta_aggr$slope      [periodi] = slope
        logger_meta_aggr$r2         [periodi] = r2
        logger_meta_aggr$rmse       [periodi] = sqrt(mean(resid^2))
        logger_meta_aggr$no_readings[periodi] = - logger_meta_aggr$no_readings[periodi] #flag as "already treated"
        
        if (!skip_existing)
        {  
          #use first and last record to supply adjacent periods with calibration data, if required
          cur_i=which(current_meta_recs_no==periodi) #rank of period currently treated
          if (cur_i < length(current_meta_recs_no)) #if this is not the last period
          {
            #browser()
            next_period = current_meta_recs_no[cur_i+1]
            if (logger_meta_aggr$no_readings[next_period] >= 0 &  #the following period hasn't been treated yet
                  logger_meta_aggr$no_readings[next_period]  < 2 ) #and lacks measurements?
            {  
              last_measurement = max(which(whole_period_i)) #last measurement of current period
              dummy_rec=meas[which(current_man_reads)[1],] #ensure same fieldnames
              dummy_rec[]=NA
              dummy_rec$gauge            = gauge
              dummy_rec$stage_datum      = datum
              dummy_rec$date             = logger_data$date [last_measurement]
              dummy_rec[c("stage","converted_stage")]= logger_data$stage_calibrated[last_measurement]
              dummy_rec$method    = "prev_period"
              dummy_rec$by        = "script"
              dummy_rec$weight..0.1.=1
              
              meas = rbind(meas, dummy_rec) #extend list of measurements by logger reading
              logger_meta_aggr$no_readings[next_period] = logger_meta_aggr$no_readings[next_period]+1 #one more measurements is available
              current_man_reads = c(current_man_reads,TRUE) #this new entry is relevant for other periods
            }  
          }  
          
          if (cur_i > 1) #if this is not the first period
          {
            #browser()
            prev_period = current_meta_recs_no[cur_i-1]
            if (logger_meta_aggr$no_readings[prev_period] >= 0 &  #the next period hasn't been treated yet
                  logger_meta_aggr$no_readings[prev_period]  < 2 ) #and lacks measurements?
            {  
              first_measurement = min(which(whole_period_i)) #first measurement of current period
              dummy_rec=meas[which(current_man_reads)[1],] #ensure same fieldnames
              dummy_rec[]=NA
              dummy_rec$gauge            = gauge
              dummy_rec$stage_datum      = datum
              dummy_rec$date             = logger_data$date [first_measurement]
              dummy_rec[c("stage","converted_stage")]= logger_data$stage_calibrated[first_measurement]
              dummy_rec$method    = "next_period"
              dummy_rec$by        = "script"
              dummy_rec$weight..0.1.=1
              
              meas = rbind(meas, dummy_rec) #extend list of measurements by logger reading
              logger_meta_aggr$no_readings[prev_period] = logger_meta_aggr$no_readings[prev_period]+1 #one more measurements is available
              current_man_reads = c(current_man_reads,TRUE) #this new entry is relevant for other periods
            }  
          }  
        }
        covered_meta_recs_no = which(current_meta_recs & logger_meta_aggr$no_readings>0) #find periods that can be corrected
        covered_meta_recs_no = covered_meta_recs_no[sort.int(logger_meta_aggr$no_readings[covered_meta_recs_no], index.return=TRUE, decreasing=TRUE)$ix] #sort by number of valid measurements
        
      }  #end while loop over possible corrections
      
      
      uncorrected = is.na(logger_meta_aggr$slope[current_meta_recs])
      if (any(uncorrected))
      {
        print(paste("",sum(uncorrected), "periods could not be corrected:", paste(which(current_meta_recs)[uncorrected], collapse=","),", applying median coefficients." ))
        mslope     = median(logger_meta_aggr$slope    [current_meta_recs][!uncorrected], na.rm=TRUE) #use median coefficients
        mintercept = median(logger_meta_aggr$intercept[current_meta_recs][!uncorrected], na.rm=TRUE)
        if (is.na(mslope))
        {
          print("median could not be computed, no correction applied.")
          browser()
          mslope=1
          mintercept=0
        }
        for (periodi in which(current_meta_recs)[uncorrected])
        {
          whole_period_i = logger_data$date >= logger_meta_aggr$Start[periodi] & logger_data$date <= logger_meta_aggr$End[periodi] #index to whole logging period
          logger_data$stage_calibrated[whole_period_i] = mslope*logger_data$stage[whole_period_i] + mintercept
          
          #log recalibration specs
          logger_meta_aggr$intercept  [periodi] = mintercept
          logger_meta_aggr$slope      [periodi] = mslope
        }  
      }  

      
      #browser()

      tt=merge(logger_meta_org[current_meta_recs_org,  !(names(logger_meta_org) %in% c("intercept", "slope", "r2", "rmse"))], logger_meta_aggr[current_meta_recs, c("logger_period", "stage_datum", "Gauge", "Device", "intercept", "slope", "r2", "rmse"  )], all.x=TRUE, all.y=FALSE )
      for (i in 1:nrow(logger_meta_aggr))
      {
        if (logger_meta_aggr$remarks[i]=="") next
        rem2update = tt$Gauge == logger_meta_aggr$Gauge[i] & 
          tt$stage_datum == logger_meta_aggr$stage_datum[i] & 
          tt$Device == logger_meta_aggr$Device[i] & 
          tt$logger_period == logger_meta_aggr$logger_period[i]   #find remarks to update
          #          tt$Start >= logger_meta_aggr$Start[i] & 
#          tt$End <= logger_meta_aggr$End[i]   #find remarks to update
        tt$remarks[rem2update]=paste(tt$remarks[rem2update], logger_meta_aggr$remarks[i], sep="/")
        
      }
      
      tt=tt[order(tt$Gauge, tt$stage_datum, tt$Device, tt$Start),] #sort records
#       tt$Start=format(tt$Start, "%d.%m.%Y %H:%M", tz="etc/GMT-1")  #restore and re-format date column
#       tt$End  =format(tt$End  , "%d.%m.%Y %H:%M", tz="etc/GMT-1")


      logger_meta_out[current_meta_recs_org, setdiff(names(logger_meta_out), c("Start", "End"))] = tt[, setdiff(names(logger_meta_out), c("Start", "End"))] #update treated records, exclude date columns
      #restore original date columns
      logger_meta_out$Start=logger_meta_out$Start_org
      logger_meta_out$End=logger_meta_out$End_org
      logger_meta_out$Start_org=NULL
      logger_meta_out$End_org=NULL
      
      #readline("saving now")      
      
      file.rename(from = logger_meta_file, to = paste0(logger_meta_file, format(Sys.time(), format="%y%m%d_%H%M"))) #create backup
      write.table(file=logger_meta_file, logger_meta_out, sep="\t", quote=FALSE, row.names=FALSE) #update logger period data
      #logger_meta_out$Start=format(logger_meta_out$Start, "%d.%m.%Y %H:%M")  #restore and re-format date column
      #logger_meta_out$End  =format(logger_meta_out$End  , "%d.%m.%Y %H:%M")
      
      logger_data$stage = logger_data$stage_calibrated
      logger_data$stage_calibrated=NULL
      
      out_of_range = NULL
      #clip measurements that are out of range
      if (gauge == "Cabecera") 
      {  #higher than logger height     
        out_of_range = logger_data$stage > 0.63 
      }  
      
      if (any(out_of_range))
      {
        print(paste0(sum(out_of_range, na.rm=TRUE)," values out of range, set to NA."))
        logger_data$stage[out_of_range] = NA
        browser()
      }
      
      
      if (write_converted_series)
      {  
        print("export adjusted series...")        
        write.table(file=paste(stage_file_dir,"corr/",gauge,"_",device,"_",datum,"_corr.txt",sep=""), 
                    data.frame(date=format(logger_data$date, "%Y-%m-%d %H:%M", tz="etc/GMT-1"), stage=logger_data$stage),
                    sep="\t", quote=FALSE, row.names=FALSE) #write updated logger data
        print("done.")        
      }
      if (do_plot!="")
      {
        plot(logger_data$date, logger_data$stage, main="recalibrated and merged", type="l")
        points(meas$date[current_man_reads],meas$converted_stage[current_man_reads], 
               col= 1+as.numeric(as.factor(meas$method[current_man_reads])),
               pch = 20 +     (meas$by         [current_man_reads]=="script"))
        
        
      }
                    
    }

#bringToTop(which = dev.cur(), stay = FALSE)

