#convert h-q records (current metering) for import in CUAHSI

rm(list=ls())

do_h0_correction = TRUE #correct for changing h0 using the parameters derived by "2_h0_correction.R"
stage_file_dir="hdata/corr/"


meas_file="qmeas_isab_hom.txt" #table with manual stage measurements (columns "date", "gauge" and  "stage_.m" required)

meas = read.table(file=meas_file, header=TRUE, sep="\t", stringsAsFactors=FALSE, na.strings=c("NA","?","na"))
names(meas)[grepl(pattern="stage_\\.",names(meas))]="stage"


if (grepl(meas$date[1], pattern = "-"))
  format="%Y-%m-%d %H:%M" else
    format="%d.%m.%Y %H:%M"
meas$date=as.POSIXct(strptime(meas$date, format=format, tz="etc/GMT-1")) #default: GMT+1
if (all(is.na(meas$date))) stop(paste0("wrong date format in ",meas_file))

unknown_time= (meas$date == round(meas$date, units="day")) | is.na(meas$date)
meas=meas[!unknown_time,] #don't use entries with lacking time stamp (i.e. date only)
meas$weight..0.1.[is.na(meas$weight..0.1.)]=1 #unset weight is interpreted as 1
meas$by[is.na(meas$by)]=""


#rename fields to match CUAHSI
  names(meas)[names(meas)=="gauge"] = "sitecode" 
  
# write file for import to CUAHSI
  source("f_write4cuahsi.R")
  
  
  
#process water stages
  meas$DataValue = meas$stage #column of actual data value
  meas$OffsetValue =  NA #default: no offset
  meas$methodc = "manual gauge reading"
  if (do_h0_correction)
    for (gauge in unique(meas$sitecode))
      for(datum in unique(meas$stage_datum))
        for(device in unique(meas$method))
        {
          current_records = (meas$sitecode == gauge) & (meas$stage_datum == datum) #select relevant records
          if (!any(current_records)) next
        
          h0_file = dir(path = paste(stage_file_dir), pattern = paste(gauge,"_.*_",datum,"_loflophases.RData",sep=""), full.names = TRUE)
          
          if (length(h0_file)>1) stop("check loflo files")
          if (length(h0_file)<1) next
          
          
          if (file.exists(h0_file))
          {
            load(file=h0_file)
            meas$OffsetValue[current_records] =lo_flo_fun(meas$date[current_records]) #h0 stage from interpolation of manually detected phases
          }
      }

  recorsd2use = (meas$weight..0.1. != 0) & (is.finite(meas$DataValue))

  write4cuahsi(cdata = meas[recorsd2use,], dest_file = paste0("CuahsiData/stage_readings.txt"), 
               varcode = "water_stage", sourcec = "SESAMII_hydro")

 
  
#process discharge meterings
  meas$varcode = "discharge"
  meas$DataValue = meas$discharge_.m..s. #column of actual data value
  meas$methodc = meas$method
  meas$OffsetValue =  NA #default: no offset
  meas$ValueAccuracy = meas$q_error...
  
  recorsd2use = (meas$weight_RC != 0) & (is.finite(meas$DataValue))
  
  write4cuahsi(cdata = meas[is.finite(meas$DataValue),], dest_file = paste0("CuahsiData/discharge_meterings.txt"), 
               varcode = "discharge", sourcec = "SESAMII_hydro")





      