#display hq-data 
#allow interactive exclusion of points
#save input file for Bayes-regression in Baratin (gauge_XY.BAD)
# do conventional rating curve and plot vs. Baratin RC

#graphics.off()
h0_drift=c("Carrasquero", "Ceguera", "Villacarli") #try to read zero-stage correction file
do_identify=FALSE #enable interactive identification of points
bayes_coeffs_file = "BaRatinAGE_v2-1_data/collected_coeffs_CI.txt"

stage_file_dir = "hdata/corr/"

#estimate rc
plot_rc <- function (rcs=c("bayes","nls")) {
  
    excluded=meas_data$weight_RC==0
    meas_data=meas_data[!excluded,]
  
    legend_items=data.frame(legend="RF",col="magenta", lty=1, stringsAsFactors=FALSE)  
    
    if ("mobforest" %in% rcs)
    {
      library(randomForest)  
      #mtry=tuneRF(x=meas_data$stage, y=meas_data$discharge, plot=TRUE)
      rc_mod=randomForest(discharge~stage, data=meas_data)
      stage_plot= seq(from=0, to=max(meas_data$stage)*1.3, length.out=30)
      
      q_pred=predict(rc_mod, newdata=data.frame(stage=stage_plot))
      
      lines(stage_plot, q_pred, col="magenta")
      clegend_items=rbind(legend_items, data.frame(legend="RF",col="magenta", lty=1))
    }
    
    if ("mobforest" %in% rcs)
    {    
    #mobforest
    library(mobForest)
  
  ## linear regression model discharge ~ stage
    mfc=mobForest_control(ntree = 50, mtry = 2, replace = TRUE, 
                          alpha = 0.05, bonferroni = TRUE, minsplit = 5)
    
    rfout <- mobForestAnalysis(as.formula(discharge ~ stage), newTestData=data.frame(stage=stage_plot, discharge=0, gauge="synth"),   
                             c("stage", "gauge"), mobForest.controls = mfc,                          , 
                             data = meas_data, processors = 6, model = linearModel)
    OOB.predMat <- getPredictedValues(rfout, newdata=TRUE)
    q_pred = OOB.predMat[,1]
    lines(stage_plot, q_pred, col="blue", lwd=2)
    lines(stage_plot, q_pred-OOB.predMat[,2], col="blue", lty="dashed", lwd=2)
    lines(stage_plot, q_pred+OOB.predMat[,2], col="blue", lty="dashed", lwd=2)
      
  ## linear regression model discharge ~ stage
    mfc=mobForest_control(ntree = 3, mtry = 2, replace = TRUE, 
                          alpha = 0.05, bonferroni = TRUE, minsplit = 5)
    
    rfout <- mobForestAnalysis(as.formula(discharge ~ stage), newTestData=data.frame(stage=stage_plot, discharge=0, gauge="synth"),   
                               c("stage", "gauge"), mobForest.controls = mfc,                          , 
                               data = meas_data, processors = 1, model = linearModel)
    OOB.predMat <- getPredictedValues(rfout, newdata=TRUE)
    q_pred = OOB.predMat[,1]
    lines(stage_plot, q_pred, col="blue")
    lines(stage_plot, q_pred-OOB.predMat[,2], col="blue", lty="dashed")
    lines(stage_plot, q_pred+OOB.predMat[,2], col="blue", lty="dashed")
    legend_items=rbind(legend_items, data.frame(legend="mobforest",col="blue", lty=1))
    
  
  #nonlinear model
    rfout <- mobForestAnalysis(as.formula(log(discharge) ~ log(stage)), newTestData=data.frame(stage=stage_plot, discharge=0, gauge="synth"),   
                               c("stage", "gauge"), mobForest.controls = mfc,                          , 
                               data = meas_data, processors = 1, model = linearModel)
    OOB.predMat <- getPredictedValues(rfout, newdata=TRUE)
    q_pred = OOB.predMat[,1]
    lines(stage_plot, exp(q_pred), col="green")
    lines(stage_plot, exp(q_pred-OOB.predMat[,2]), col="green", lty="dashed")
    lines(stage_plot, exp(q_pred+OOB.predMat[,2]), col="green", lty="dashed")
    legend_items=rbind(legend_items, data.frame(legend="mobforest, linearized",col="green", lty=1))
    
    
  #glm
    rfout <- mobForestAnalysis(as.formula(discharge ~ stage), newTestData=data.frame(stage=stage_plot, discharge=0, gauge="synth"), family=poisson(),  
                             c("stage", "gauge"), mobForest.controls = mfc,                          , 
                             data = meas_data, processors = 6, model = glinearModel)
  OOB.predMat <- getPredictedValues(rfout, newdata=TRUE)
  q_pred = OOB.predMat[,1]
  lines(stage_plot, q_pred, col="red")
  lines(stage_plot, q_pred-OOB.predMat[,2], col="red", lty="dashed")
  lines(stage_plot, q_pred+OOB.predMat[,2], col="red", lty="dashed")
  legend_items=rbind(legend_items, data.frame(legend="mobforest, glm",col="red", lty=1))
    
}
  if (("bayes" %in% rcs) & exists("bayes_coeffs") & (gauge %in% bayes_coeffs$gauge)) 
  {
    # Bayes curve approximated by coefficients
        # g_row= bayes_coeffs$gauge == paste(gauge, sep="")
    # q_pred = bayes_coeffs$C[g_row] * (stage_plot-bayes_coeffs$h0[g_row])^bayes_coeffs$b[g_row]
    # lines(stage_plot, q_pred, col="orange")

    # origfinal data from baratin
    bayes_table=read.table(paste0("BaRatinAGE_v2-1_data/", gauge,"/rc.csv"), header=TRUE, sep=";", stringsAsFactors = FALSE)

    lines(bayes_table[,1], bayes_table[,3], col="orange", lty="dotted")
    lines(bayes_table[,1], bayes_table[,5], col="orange", lty="dashed")
    lines(bayes_table[,1], bayes_table[,6], col="orange", lty="dashed")
    legend_items=rbind(legend_items, data.frame(legend="bayes",col="orange", lty=1))
  }
    
  if ("nls" %in% rcs) 
  {
 #   browser()
    pos_stage_i=meas_data$stage > 0 #use only positive stage readings for prior estimation (I)
    
    lm_guess=lm(log(discharge)~log(stage), data=meas_data[pos_stage_i,], weights = meas_data$weight_RC[pos_stage_i])
    init_c=as.numeric(exp(coef(lm_guess)[1]))
    init_b=as.numeric(    coef(lm_guess)[2])
    
    
    #prior estimation (II)
    nls_mod0 = nls(formula=discharge~C*stage^b, data=meas_data, start=c(C=init_c*2, b=init_b*2), subset=pos_stage_i, algorithm="port",
                control=nls.control(maxiter=3000, warnOnly=TRUE), weights = meas_data$weight_RC)  #no offset-estimation
    cc = coef(nls_mod0)["C"]
    bb = coef(nls_mod0)["b"]
    h0 = min(c(0, min(meas_data$stage)))
    
#     nls_mod=nls(formula=discharge~C*(abs(stage-h0))^b, data=meas_data, start=c(cc, h0=h0, bb), algorithm="port",
#                 control=nls.control(maxiter=3000, warnOnly=TRUE), weights = meas_data$weight_RC)
    
    nls_mod=nls(formula=discharge~C*(abs(stage-h0))^b, data=meas_data, 
                start=c(cc,   h0=h0, bb), 
                lower=c(C=1e-3, h0=h0-10, b=1e-3), 
                upper=c(C=1e3,  h0=min(meas_data$stage) , b=10), 
                algorithm="port",
                control=nls.control(maxiter=3000, warnOnly=TRUE), weights = meas_data$weight_RC)
    
    
    if(!nls_mod$convInfo$isConv)
      print(paste("no convergence for",gauge)) else
    {
      cc = coef(nls_mod)["C"]
      bb = coef(nls_mod)["b"]
      h0 = coef(nls_mod)["h0"]
    }    
    write.table(file="nls_rq.txt", data.frame(gauge=gauge, C=cc, b=bb,	h0=h0), append=TRUE, sep="\t", row.names=FALSE, col.names=!file.exists("nls_rq.txt"), quote=FALSE)
    q_pred = cc * (stage_plot-h0)^bb
    rmse = sqrt(mean(resid(nls_mod)^2))
    r2 = 1 - var(resid(nls_mod)) / var(meas_data$discharge)
#    browser()
    lines(stage_plot, q_pred, col="black")
    legend_items=rbind(legend_items, data.frame(legend=paste0("non-lin fit:",format(r2, digits=3)),col="black", lty=1))
    
  }  
  legend_items=legend_items[-1,]
  return(legend_items)
}


#plot_rc()


##measured data
meas_file="qmeas_isab_hom.txt" #table with homogenized measurements
meas = read.table(file=meas_file, header=TRUE, sep="\t", stringsAsFactors=FALSE, na.strings=c("NA","?","na"))
names(meas)[grepl(pattern="discharge",names(meas))]="discharge"
names(meas)[grepl(pattern="stage_\\.",names(meas))]="stage"
meas$weight_RC[is.na(meas$weight_RC)]=1 #unset weight is interpreted as 1
#meas=na.omit(meas)

if (grepl(meas$date[1], pattern = "-"))
  format="%Y-%m-%d %H:%M" else
  format="%d.%m.%Y %H:%M"
meas$date = as.POSIXct(strptime(meas$date, format=format, tz="etc/GMT-1"))

#meas = meas[meas$gauge=="Villacarli",]

#bayes_linreg coefficients, if present
bayes_coeffs=read.table(bayes_coeffs_file, header=TRUE)
bayes_coeffs$gauge = sub(bayes_coeffs$gauge, pattern = "_m$", repl="")


for (gauge in unique (meas$gauge))
{
  windows()
  valid_rows=meas$gauge==gauge & !is.na(meas$stage + meas$discharge)
  meas_data=meas[valid_rows,]
  print(paste(gauge,", #measurements :",sum(valid_rows)))
  
  use_h0_correction = gauge %in% h0_drift
  if (use_h0_correction) #apply h0-correction, if present
  {
    h0_file = dir(path = paste(stage_file_dir), pattern = paste(gauge,"_.*_loflophases.RData",sep=""), full.names = TRUE)
    
    if (length(h0_file)>1) stop("check: multiple loflo files found")
    if (length(h0_file)<1) next
    
    if (file.exists(h0_file))
    {
      load(file=h0_file)

      find_closest=function(series, target) #find closest match of target in series
      {return(which.min(abs(series-target))[1])}
    
      h0 = lo_flo_fun(as.numeric(meas_data$date)) 
      #replace uncovered periods with closest h0 corr
      no_h0_corr = is.na(h0) #no correction of zero flow for these measurements, because they were outside correction period
      extrapolation = sapply(find_closest, X = meas_data$date[no_h0_corr], series=meas_data$date[!no_h0_corr])
      if (any(extrapolation))
      {  
        h0[no_h0_corr] = h0[!no_h0_corr][extrapolation]     
        meas_data$by[no_h0_corr] = paste0(meas_data$by[no_h0_corr],"_h0xpolated")
        meas_data$weight_RC[no_h0_corr] = 0
      }
      
      #second version: eexclude measurements outside period of h0-correction
          meas_data$weight_RC[meas_data$date < min(lf_phase$date_begin)] = 0
      
      
      
      meas_data$stage =  meas_data$stage-h0
  
      
      meas_data= meas_data[!is.na(meas_data$stage), ] 
      print(paste(gauge,", #measurements within low-flow span:", sum(meas_data$weight_RC!=0)))
      
      
      h0_corr="-h0corr"
      gauge=paste0(gauge,"-h0corr")
      
    }
  } else h0_corr=""
  
  
  stage_plot= seq(from=0, to=max(meas_data$stage)*1.3, length.out=30)
  discharge_plot = seq(from=0, to=max(meas_data$discharge)*1.3, length.out=30)

  plot(stage_plot, discharge_plot, xlim=c(0, max(stage_plot)), type="n", main=paste0(gauge," (n=",sum(meas_data$weight_RC!=0)," of ", nrow(meas_data),")"))
  points (meas_data$stage, meas_data$discharge, col=as.factor(meas_data$by), pch=20)
  excluded=meas_data$weight_RC==0
  points (meas_data$stage[excluded], meas_data$discharge[excluded], pch="o") #mark excluded points
    
  legend("topleft", legend=unique(meas_data$by), col=unique(as.factor(meas_data$by)), pch=20)
  legend_items = plot_rc(rcs=c("bayes","nls"))
  #legend_items=plot_rc(rcs=c("nls"))
  legend("bottomright", legend=legend_items$legend, col=legend_items$col, lty=legend_items$lty)
  
  
  if (do_identify) 
  {  
    print("click points to attach date label")
    sel=identify(meas_data$stage, meas_data$discharge, label=meas_data$date)
    #sel=identify(meas_data$stage, meas_data$discharge, label=meas_data$by)
    print(meas_data$date[sel])
    
  }  

  meas_data= meas_data[!excluded,]
  if (exists("h0"))
    h0       = h0[!excluded]
  savePlot(filename=paste("rc_plots/",gauge,"_rc.emf",sep=""), type = "emf")
  
  #write XY.BAD for import to Baratin
    write.table(file=paste(gauge,"_XY.BAD",sep=""), cbind("H", "sigmaH", "Q", "sigmaQ"),
                sep=" ", quote=FALSE, row.names=FALSE, col.names=FALSE)  
    write.table(file=paste(gauge,"_XY.BAD",sep=""), cbind(meas_data$stage, ifelse(use_h0_correction, 10, 1)*0.005, meas_data$discharge, meas_data$discharge*meas_data$q_error.../100/2), sep=" ", quote=FALSE, row.names=FALSE, append=TRUE, col.names=FALSE)
  
  
}
