#convert discahrge series generated by Baratin to files for CUAHSI-import

base_dir="./BaRatinAGE_v2-1_data/"    #baratin data directory


gauge_dirs = c("Cabecera", "Carrasquero-h0corr", "Ceguera-h0corr", "Lascuarre", "Villacarli") 


library(RHydro)

for (gauge_dir in gauge_dirs[3])
{
  if (!file.exists(paste0(base_dir, gauge_dir))) next
  
  q_data = read.table(paste0(base_dir, gauge_dir,"/q.txt"), header=TRUE, sep=";", stringsAsFactors = FALSE)
  summary(q_data)
  q_data$X=NULL
  
  if (gauge_dir == "Cabecera") 
  {  #higher than logger height     
    out_of_range = q_data$Qmaxpost.m3.s. > 26.9 
    q_data$Qmaxpost.m3.s.[out_of_range] = NA
    q_data$Qlow_stage.m3.s.[out_of_range] = NA
    q_data$Qhigh_stage.m3.s.[out_of_range] = NA
    q_data$Qlow_stage.param.m3.s.[out_of_range] = NA
    q_data$Qhigh_stage.param.m3.s.[out_of_range] = NA
    q_data$Qlow_total.m3.s.[out_of_range] = NA
    q_data$Qhigh_total.m3.s.[out_of_range] = NA
  }  
  

  if (gauge_dir == "Lascuarre") 
  {  #negative values at extreme low flow: set to 0     
    q_data$Qmaxpost.m3.s.          = pmax(q_data$Qmaxpost.m3.s., 0)
    q_data$Qlow_stage.m3.s.        = pmax(q_data$Qlow_stage.m3.s., 0)
    q_data$Qhigh_stage.m3.s.       = pmax(q_data$Qhigh_stage.m3.s., 0)
    q_data$Qlow_stage.param.m3.s.  = pmax(q_data$Qlow_stage.param.m3.s., 0)
    q_data$Qhigh_stage.param.m3.s. = pmax(q_data$Qhigh_stage.param.m3.s., 0)
    q_data$Qlow_total.m3.s.        = pmax(q_data$Qlow_total.m3.s., 0)
    q_data$Qhigh_total.m3.s.       = pmax(q_data$Qhigh_total.m3.s., 0)
    
  }  
  
    
  
  
  #smoothen steps
  q_data$Qmaxpost.m3.s.    = remove_plateaus(runoff=q_data$Qmaxpost.m3.s.,    method="center")
  q_data$Qhigh_total.m3.s. = remove_plateaus(runoff=q_data$Qhigh_total.m3.s., method="center")
  q_data$Qlow_total.m3.s.  = remove_plateaus(runoff=q_data$Qlow_total.m3.s. , method="center")
  
  names(q_data)[names(q_data)=="Qmaxpost.m3.s."] = "DataValue"    #rename column that contains the actual data
  
  source("f_write4cuahsi.R")
  
  q_data$ValueAccuracy = (q_data$Qhigh_stage.param.m3.s. - q_data$Qlow_stage.param.m3.s.) / 2 #define accuracy
  q_data$date          = date=ISOdatetime(year = q_data$Year, month = q_data$Month, day = q_data$Day,
                                          hour = q_data$Hour, min   = q_data$Min,   sec = q_data$Sec, tz = "Etc/GMT-1")
  
   write4cuahsi(cdata = q_data, dest_file = paste0("CuahsiData/",gauge_dir,"_q.txt"), 
                varcode = "discharge", sitecode <- gauge_dir, sourcec = "SESAMII_hydro", methodc="baratin_rating_curve")
   
   
}  
  
  


