#read logger records from multiple files of specified gauge, device and datum
read_loggerdata = function(gauge, datum, device, stage_file_dir="./", do_correct_tz=TRUE) #try to read logger data
{
  logger_meta = read_logger_meta(logger_meta_file = "observation_periods.txt")

  pot_file = dir(path=stage_file_dir, pattern=paste0(paste(gauge,device,datum,sep="_"),".*txt$"))
  pot_file = pot_file[!grepl(pot_file, pattern = "_h0\\.txt")]  #ignore possibly zero-stage files
  if (length(pot_file)==0) return(NULL)
  logger_data=data.frame()
  for (ifile in pot_file)
  {
    print(paste("reading", ifile))
    #browser()
    first_line=read.table(paste(stage_file_dir,ifile,sep=""), sep="\t", nrows=1, stringsAsFactors=FALSE, strip.white=TRUE)
    
    options(warn=-1)
      tt=all(is.na(sapply(X=sub(first_line[1,], pattern = ",", repl="."), FUN=                 as.numeric)))
    options(warn=0)
    if (tt) 
    {  
      header=TRUE
      first_line=read.table(paste(stage_file_dir,ifile,sep=""), sep="\t", nrows=1, skip=1)
    }  else header=FALSE
    if (any(grepl(pattern=",", x=first_line))) dec="," else
      dec="."
    logger_cont=read.table(paste(stage_file_dir,ifile,sep=""), sep="\t", header=header, dec=dec, stringsAsFactors=FALSE, na.strings="NA", strip.white=TRUE, blank.lines.skip = TRUE)
    
    
    for (i in rev(1:ncol(logger_cont))) #remove nodata columns
      if (all(is.na(logger_cont[,i])))
        logger_cont=logger_cont[,-i]
#     for (i in rev(1:nrow(logger_cont))) #remove nodata rows
#       if (all(is.na(logger_cont[i,]) | (logger_cont[i,]=="")))
#         logger_cont=logger_cont[-i,]
    
    #figure out date format
    if (grepl("\\.", x=logger_cont[1,1]))
    {  
      if (grepl("\\.", x=substr(x=logger_cont[1,1], start=1, stop=3)))
        format="%d.%m.%Y" else
          format="%Y.%m.%d"
    }  else
      format="%Y-%m-%d"
    if (grepl(":", x=logger_cont[1,2])) #time in second column, merge with first
    {  
      logger_cont[,1]=paste(logger_cont[,1], logger_cont[,2], sep=" ")
      logger_cont[,2]=NULL
    }
    if (grepl("\\:[0-9]{2}:[0-9]{2}", x=logger_cont[1,1]))
      format = paste(format, "%H:%M:%S") else
        format = paste(format, "%H:%M") 
    logger_cont[,1]=as.POSIXct(strptime(logger_cont[,1], format=format, tz="etc/GMT-1")) #interprete columns as GMT+1 (!)
    #browser()
    NA_dates=is.na(logger_cont[,1])
    NA_data = is.na(logger_cont[,2])
    blanklines=NA_dates & NA_data
    logger_cont = logger_cont[!blanklines,] #ignore blank lines
    NA_dates[blanklines]=FALSE
    
    if (any(NA_dates))
      stop(paste("unreadable dates found in",ifile,", line(s)",paste(which(NA_dates), collapse=",")))
    if (ncol(logger_cont)>2)
      logger_cont=logger_cont[, c(1,3)] #only keep first and third column
    #      logger_cont=logger_cont[,c(1,ncol(logger_cont))] #only keep last and first column
    names(logger_cont)=c("date","stage")
    #if (median(logger_cont$stage, na.rm=TRUE) >10)
    if (device=="TT")
      logger_cont$stage=logger_cont$stage/1000  #convert mm to m
    
    #browser()
    #correct (daylight saving) the given input values in "date" according to the information in "obs_period_file"
    if (do_correct_tz)
    logger_cont$date = correct_tz(date=logger_cont$date, gauge, datum, device, logger_meta)
    
    logger_data=rbind(logger_data, logger_cont)
  }
  #logger_data=logger_data[sort.int(logger_data$date, index.return=TRUE)$ix,]
  

  return(logger_data)
}

read_logger_meta=function(logger_meta_file="observation_periods.txt")
{
  logger_meta = read.table(file=logger_meta_file, header=TRUE, sep="\t", stringsAsFactors=FALSE, na.strings=c("NA","?","na"))
  if (grepl("\\.", logger_meta$Start[1])) format="%d.%m.%Y %H:%M" else
    format="%Y-%m-%d %H:%M"
  logger_meta$Start_org=logger_meta$Start #keep these for later use
  logger_meta$End_org=logger_meta$End
  
  logger_meta$Start=as.POSIXct(strptime(logger_meta$Start, format=format, tz="etc/GMT-1")) #default: GMT+1
  logger_meta$End  =as.POSIXct(strptime(logger_meta$End,   format=format, tz="etc/GMT-1"))
  
  t_offset_h= as.numeric(sub(logger_meta$time,pattern = "GMT", repl=""))-1
  
  logger_meta$Start = logger_meta$Start - t_offset_h * 3600 #convert to GMT+1
  logger_meta$End   = logger_meta$End   - t_offset_h * 3600 #convert to GMT+1
  #logger_meta$time = "GMT+1" #don't activate this line, we still need this information

  #check validity
  err = (logger_meta$Start >= logger_meta$End)
  if (any(err)) stop(paste("Start/end information reversed in line(s)",paste(which(err), collapse=", "),"of observation periods"))
  
  logger_meta = logger_meta[order(logger_meta$Gauge, logger_meta$Device, logger_meta$stage_datum, logger_meta$Start),]
  
  #check for overlap in adjacent periods
  for (gauge in unique(logger_meta$Gauge))
    for (datum in unique(logger_meta$stage_datum))
      for (device in unique(logger_meta$Device))
      {
        current_meta_recs=logger_meta$Gauge==gauge &
        logger_meta$stage_datum==datum &
        logger_meta$Device     ==device #meta records of current gauge
        current_meta_recs_no = which(current_meta_recs) #convert to IDs (i.e. line numbers)
  
        if (length(current_meta_recs_no)==0) next
  
        tdiffs= difftime(logger_meta$Start[current_meta_recs][-1], logger_meta$End[current_meta_recs][-sum(current_meta_recs)], units="h")
        err =   c(tdiffs < 0, FALSE) #add FALSE, because the last entry cannot be compared to another one
        
        if (any(err)) stop(paste("End of observation period overlaps with next start in line(s)",paste(current_meta_recs_no[err]+1, collapse=", "),"of observation periods"))
      }
  
  
  return(logger_meta)
}

correct_tz=function(date, gauge, datum, device, logger_meta)
{
#correct (daylight saving) the given input values in "date" according to the information in "logger_meta"
 #browser()
  current_meta_recs=logger_meta$Gauge==gauge &
    logger_meta$stage_datum==datum &
    logger_meta$Device     ==device #meta records of current gauge
  current_meta_recs_no = which(current_meta_recs) #convert to IDs (i.e. line numbers)
  
  if (length(current_meta_recs_no)==0) stop(paste0("no observation periods found for ", gauge,"/",datum,"/",device))
  
  breaks = which(diff(date)<0) #find points of time overlap due to daylight saving
  if (any(breaks))
    warning("Strange time breaks. Omnilog export error (check midnight timestamps), true data gaps or unduely stitched files (", paste0(date[breaks], collapse=", ", ")"))
  
  time_series_center=mean(date)
  #time_series_center=as.POSIXct(format(time_series_center, tz="GMT"), tz="etc/GMT-1") #clumsy way of converting to GMT+1 (!) to avoid warnings in next step
  
  cur_period = logger_meta$Start[current_meta_recs_no] <= time_series_center &  #robust way of finding corresponding logger period
               logger_meta$End  [current_meta_recs_no] >= time_series_center 

  if (sum(cur_period)== 0)
    stop(paste0("File with dates ", min(date), " - ", max(date), " has no corresponding logger period."))
  
  if (sum(cur_period)> 1)
    stop(paste0("File with dates ", min(date), " - ", max(date), " spans more than a single period, split it"))
  
   #correct timespan for tz-attribute (add/substract hours from currently GMT+1)
     shift = as.numeric(sub(logger_meta$time[current_meta_recs_no][cur_period], pattern="GMT", repl="")) - 1
     date = date - shift*60*60
     
     attr(date,"tzone")="etc/GMT-1" #quick way setting timezone to GMT+1 (!) to avoid warnings in next step
     #date = as.POSIXct(format(date, tz="etc/GMT-1"), tz="etc/GMT-1") #clumsy and slow way of converting to GMT+1 (!) to avoid warnings in next step

     outside = date < logger_meta$Start[current_meta_recs_no][cur_period] |  
               date > logger_meta$End  [current_meta_recs_no][cur_period] 
     if (any(outside) )
     {  
       print(paste(sum(outside),"records found outside logger periods. These were not timecorrected, please check logger periods."))
       print(head(date[outside]))
       stop()
       browser()
     }     
     
  return(date)
  
  
}