#homogenize rating curve data
#read raw ratings from raw_meas (e.g. "qmeas_isab.txt") and converts them to common datum (standard datum)
#write output file (qmeas_isab_hom.txt)

raw_meas_file="qmeas_isab.txt" #table with raw measurements
convert_file="gauge_datum_conv.txt" #to be saved as txt from p:/sesam_data/SESAMII/Data_Fieldwork/Spain/Discharge_Gauges/Discharge/gauge_datum.xlsx
                                    #and reformatted using convert_stage.R

convert_info  = read.table(fill=TRUE,file=convert_file, header=TRUE, sep="\t", stringsAsFactors=FALSE)

raw_meas = read.table(file=raw_meas_file, header=TRUE, sep="\t", stringsAsFactors=FALSE, na.strings=c("NA","?","na"))

raw_meas$converted_from=""

#guess timezone, if not specified
  tz_guessed = as.POSIXct(strptime(raw_meas$date, format="%d.%m.%Y %H:%M", tz="CET"))
  tz_guessed = sub(pattern="1 ", replacement="", strftime(tz_guessed,"1", usetz=TRUE)) #timezone guessed from local setting at time of raw_measurement
  tz_guessed[tz_guessed=="CET"]  = "GMT+1"
  tz_guessed[tz_guessed=="CEST"] = "GMT+2"
  unknown_tz = raw_meas$timezone  ==""
  raw_meas$timezone[unknown_tz] = tz_guessed[unknown_tz]  #set timezones which are still unknown

#correct timezone    
  raw_meas$date=as.POSIXct(strptime(raw_meas$date, format="%d.%m.%Y %H:%M", tz="etc/GMT-1")) #default: GMT+1
  raw_meas$date[raw_meas$timezone=="GMT+2"] = raw_meas$date[raw_meas$timezone=="GMT+2"] - 3600 #convert to GMT+1
  raw_meas$timezone="GMT+1" #now all are GMT+1
  raw_meas = raw_meas[!is.na(raw_meas$stage_.m.) | !is.na(raw_meas$date),]  #remove NA-values for stage
  
  

#prepare datum conversion
source("../harmonize_stagerecords/convert_stage.R")


for (gauge in unique (raw_meas$gauge))
{
  valid_rows=raw_meas$gauge==gauge
  print(paste("gauge", gauge, ":", sum(valid_rows), "stage measurements."))
  print(paste(" ", sum(valid_rows & is.finite(raw_meas$discharge_.m..s.) & is.finite(raw_meas$stage_.m.)), "discharge measurements."))
  
  standard_datum =  as.character(convert_info$from_datum[convert_info$remarks=="standard" & convert_info$gauge==gauge])
  
  dates_in_standard = raw_meas$date[valid_rows][raw_meas$stage_datum[valid_rows]==standard_datum]
  #get dates for which measurements in the standard datum exist
  redundant_meas    = valid_rows & (raw_meas$date %in% dates_in_standard) & (raw_meas$stage_datum!=standard_datum)
  #tt=which(valid_rows)[!redundant_meas] #remove redundant entries
  valid_rows=valid_rows & !redundant_meas
  
  
  converted_stages =    #convert all available readings into current datum
    convert_stage(gauge=gauge, convert_info=convert_info, 
                  from_datum=raw_meas$stage_datum[valid_rows],
                  to_datum=standard_datum,
                  stage=raw_meas$stage_.m.[valid_rows])
  conv_failed_i = is.finite(raw_meas$stage_.m.[valid_rows]) & !is.finite(converted_stages)
  if(any(conv_failed_i))
    print(paste(" record(s)", paste((1:nrow(raw_meas))[valid_rows][conv_failed_i], collapse=", "), "could not be converted."))
  
  raw_meas$stage_.m.[valid_rows] = converted_stages
  
  raw_meas$converted_from[valid_rows] = raw_meas$stage_datum[valid_rows]
  raw_meas$stage_datum[valid_rows]= standard_datum
  print(paste(" ", sum(valid_rows & is.finite(raw_meas$discharge_.m..s. + raw_meas$stage_.m.)), "valid discharge measurements."))
  raw_meas = raw_meas[!redundant_meas,] #remove redundant entries (stage reading at the same date at non-standard datum)  
}

write.table(file=paste(sub("\\..*$","", raw_meas_file),"_hom.txt",sep=""), raw_meas, quote=FALSE, row.names=FALSE, sep="\t")
