+++

# SAIH data for CUASHI

+++

setwd("U:/HiWi_ER/CUAHSI/SAIH/")


### Temperature: Data reading and processing####

temp_saih <- function(single_file){

temp <- read.table(single_file , stringsAsFactors = F, sep=";", dec= ",", skip = 1, na.strings = c("NO DATO","-99999")) ;
colnames(temp) <- c("Tag","Fecha","Valor")

temp$Valor[which(temp$Valor < -1000)] = NA # didnt work with na.strings() when reading table
temp$Valor[which(temp$Valor ==  -30)] = NA # several values -30 in data; no real value

temp$posixdate = as.POSIXct(strptime(temp$Fecha, format="%d/%m/%y %H:%M", tz="etc/GMT-1"))
temp$posixdate_CET  = as.POSIXct(strptime(temp$Fecha, format="%d/%m/%y %H:%M", tz="CET"))


# in year 2006 several periods in Summer continuously recorded 0C
# if longer than 2h on 0,00C made NA

sub1 <- subset(temp, format(posixdate, "%Y") <"2008")
sub2 <- subset(temp, format(posixdate, "%Y") >="2008")
#sub1[which(sub1$Valor == 0),]

if(nrow(sub1)>0){

sub1.dif <- diff(sub1$Valor)
sub1.dif[is.na(sub1.dif)]=0

period_len = 0*sub1.dif

wl = length(sub1.dif)
i = 1

while (i <= wl)
{
  if (sub1.dif[i] != 0)
  {  
    period_len[i]=0
    i = i+1
    next
  }
  pl = which(sub1.dif[i:wl] != 0)[1] - 1
  if (is.na(pl)) pl=wl-i
  period_len[i:(i+pl)]=pl
  i = i + pl+1
}

period_len[wl+1] = NA
zero_period  = period_len >= 8 & sub1$Valor == 0
long_period = period_len >= 4*24 #if for more than 1-day no change in recorded temperature to NA; the case in 2006 P030
sub1$zero_period  = zero_period
sub1$long_period = long_period

#if(nrow(sub1) >0){
#  plot(sub1$posixdate, sub1$Valor, ylab = "Temperature C", xlab= "Time", type="l")
#  points(sub1$posixdate[which(sub1$zero_period == T)], sub1$Valor[which(sub1$zero_period == T)],col=2, pch=19)
#  legend("topleft", c("Range", as.character(round(range(sub1$Valor, na.rm=T), digits = 1))))}

if(length(which(sub1$zero_period == T)) > 0){
   sub1$Valor[which(sub1$zero_period == T)] = NA}
sub1$zero_period = NULL

if(length(which(sub1$long_period == T)) > 0){
   sub1$Valor[which(sub1$long_period == T)] = NA}
sub1$long_period = NULL

sub1.dif <- diff(sub1$Valor)
sub1.dif[is.na(sub1.dif)]=0


#in year 2006/2007 further irregularities; jumps to very negative values; one-value jumps
#if jump >10C made NA
sub1.dif[length(sub1.dif)+1]=NA
sub1 <-sub1[-which(sub1.dif > 10),]

temp <-  rbind(sub1, sub2)

}

#from 2013 on switch to summer time in data
temp$posixdate[which(format(temp$posixdate_CET, "%Y") > "2012")] <-  temp$posixdate_CET[which(format(temp$posixdate_CET, "%Y") > "2012")]
#date automatically adjusted and converted to GMT+1

sorted_ix=sort.int(temp$posixdate, index.return = TRUE)$ix
temp = temp[sorted_ix,]

filename <- temp[1,1]

temp <- data.frame(Tag=temp$Tag, posixdate=temp$posixdate, Valor=temp$Valor)


# Further data quality management
if(single_file == "P032_S17_temp_2016_12_01.txt"){
  #error in data: single value of 12.0C in raw data; whole day around 29C
  #see also station comparison P032_P102_old
  temp$Valor[which(as.character(temp$posixdate) == "2015-07-09 15:15:00")] = NA}

if(single_file == "A258_B01_temp_2016_12_01.txt"){
  #single value 0C; values around all ~13C
  temp$Valor[which(as.character(temp$posixdate) == "2006-08-31 09:45:00")] = NA}

if(single_file == "P030_B01_temp_2016_12_01.txt"){
  #before NA-period in august 2007 already strong jumps in data
  #8h before also set to NA; no reliable data
  temp$Valor[which(as.character(temp$posixdate) == "2007-08-07 08:00:00"):which(as.character(temp$posixdate) == "2007-08-07 15:15:00")] = NA}


save(file = paste0("R_Data/",filename,".RData"), list="temp")

sub2006 <- subset(temp, format(posixdate, "%Y")=="2006")
sub2007 <- subset(temp, format(posixdate, "%Y")=="2007")
sub2008 <- subset(temp, format(posixdate, "%Y")=="2008")
sub2009 <- subset(temp, format(posixdate, "%Y")=="2009")
sub2010 <- subset(temp, format(posixdate, "%Y")=="2010")
sub2011 <- subset(temp, format(posixdate, "%Y")=="2011")
sub2012 <- subset(temp, format(posixdate, "%Y")=="2012")
sub2013 <- subset(temp, format(posixdate, "%Y")=="2013")
sub2014 <- subset(temp, format(posixdate, "%Y")=="2014")
sub2015 <- subset(temp, format(posixdate, "%Y")=="2015")
sub2016 <- subset(temp, format(posixdate, "%Y")=="2016")

pdf(paste("plots/",filename,".pdf",sep = ""),width=10,height=8)

if(nrow(temp) >0){
  plot(temp$posixdate, temp$Valor, main=paste(filename), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(temp$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2006) >0){
  plot(sub2006$posixdate, sub2006$Valor, main=paste(filename,"_2006",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2006$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2007) >0){
  plot(sub2007$posixdate, sub2007$Valor, main=paste(filename,"_2007",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2007$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2008) >0){
  plot(sub2008$posixdate, sub2008$Valor, main=paste(filename,"_2008",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2008$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2009) >0){
  plot(sub2009$posixdate, sub2009$Valor, main=paste(filename,"_2009",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2009$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2010) >0){
  plot(sub2010$posixdate, sub2010$Valor, main=paste(filename,"_2010",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2010$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2011) >0){
  plot(sub2011$posixdate, sub2011$Valor, main=paste(filename,"_2011",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2011$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2012) >0){
  plot(sub2012$posixdate, sub2012$Valor, main=paste(filename,"_2012",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2012$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2013) >0){
  plot(sub2013$posixdate, sub2013$Valor, main=paste(filename,"_2013",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2013$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2014) >0){
  plot(sub2014$posixdate, sub2014$Valor, main=paste(filename,"_2014",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2014$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2015) >0){
  plot(sub2015$posixdate, sub2015$Valor, main=paste(filename,"_2015",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2015$Valor, na.rm=T), digits = 1))))}

if(nrow(sub2016) >0){
  plot(sub2016$posixdate, sub2016$Valor, main=paste(filename,"_2016",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2016$Valor, na.rm=T), digits = 1))))}

jump <- data.frame(2006:2016,rep(NA,11))
if(length(temp$posixdate[which(temp$posixdate == "2006-03-26 01:45:00")]) > 0){
  jump[1,2] <- temp$posixdate[which(temp$posixdate == "2006-03-26 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2006-03-26 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2007-03-25 01:45:00")]) > 0){
  jump[2,2] <- temp$posixdate[which(temp$posixdate == "2007-03-25 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2007-03-25 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2008-03-30 01:45:00")]) > 0){
  jump[3,2] <- temp$posixdate[which(temp$posixdate == "2008-03-30 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2008-03-30 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2009-03-29 01:45:00")]) > 0){
  jump[4,2] <- temp$posixdate[which(temp$posixdate == "2009-03-29 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2009-03-29 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2010-03-28 01:45:00")]) > 0){
  jump[5,2] <- temp$posixdate[which(temp$posixdate == "2010-03-28 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2010-03-28 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2011-03-27 01:45:00")]) > 0){
  jump[6,2] <- temp$posixdate[which(temp$posixdate == "2011-03-27 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2011-03-27 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2012-03-25 01:45:00")]) > 0){
  jump[7,2] <- temp$posixdate[which(temp$posixdate == "2012-03-25 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2012-03-25 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2013-03-31 01:45:00")]) > 0){
  jump[8,2] <- temp$posixdate[which(temp$posixdate == "2013-03-31 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2013-03-31 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2014-03-30 01:45:00")]) > 0){
  jump[9,2] <- temp$posixdate[which(temp$posixdate == "2014-03-30 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2014-03-30 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2015-03-29 01:45:00")]) > 0){
  jump[10,2] <- temp$posixdate[which(temp$posixdate == "2015-03-29 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2015-03-29 01:45:00")]}
if(length(temp$posixdate[which(temp$posixdate == "2016-03-27 01:45:00")]) > 0){
  jump[11,2] <- temp$posixdate[which(temp$posixdate == "2016-03-27 01:45:00")+1] - temp$posixdate[which(temp$posixdate == "2016-03-27 01:45:00")]}

plot(jump, pch=19,main=paste("Winter time to Summer time ",filename,sep = ""), xlab="Year", ylab="Time between 01:45 and next time step")

#Dublicate check
if(anyDuplicated(temp$posixdate)>0){duplicates_file = (paste("Dublicates in ", single_file))
} else {duplicates_file = paste("No dublicates in ", single_file)}

if(nrow(temp) >0){
  plot(temp$posixdate, temp$Valor, main=paste(filename),ylab="", xlab="", axes=F, type="n")
  legend("center", paste(duplicates_file), bg="grey90")}

#Time difference
diff_time <- diff(temp$posixdate)
diff_time[which(as.numeric(diff_time) == 15)] <- NA
plot(temp$posixdate[1:(length(temp$posixdate)-1)],diff_time, ylim=c(0,max(as.numeric(diff_time), na.rm=T)),pch=19, xlab="Date", ylab="Time difference [min]", main="Time Difference not 15min")

dev.off()

}


dir()
single_file <-   "A145_B01_temp_2016_12_01.txt"

temp_saih(single_file)


### Temperature: Quality control####

load("R_Data/A145B01TEMPE.RData") ; A145 <- temp
load("R_Data/A258B01TEMPE.RData") ; A258 <- temp
load("R_Data/EM47G02TEMPE.RData") ; EM47 <- temp
load("R_Data/EM48B02TEMPE.RData") ; EM48 <- temp
load("R_Data/P029G01TEMPE.RData") ; P029 <- temp
load("R_Data/P030B01TEMPE.RData") ; P030 <- temp
load("R_Data/P032S17TEMPE.RData") ; P032 <- temp
load("R_Data/P102Z02TEMPE.RData") ; P102 <- temp


temp_func <- function(data_1, data_2){
  
  pdf(paste("plots/station_compare_temp.pdf",sep = ""),width=10,height=8)
  
  merge_data = merge(data_1, data_2, by = "posixdate", all = FALSE) 
  
  hist(merge_data$Valor.x - merge_data$Valor.y, col=2, breaks=50)
  
  plot(merge_data$Valor.x, merge_data$Valor.y, pch=19, xlab = "data_1", ylab = "data_2", main = "Station comparison", 
       xlim = c(min(merge_data[,c(3,5)], na.rm = T),max(merge_data[,c(3,5)], na.rm = T)),
       ylim = c(min(merge_data[,c(3,5)], na.rm = T),max(merge_data[,c(3,5)], na.rm = T))
  )
  #abline(-mean(merge_data$Valor.x - merge_data$Valor.y, na.rm = T),1, col="red",lwd=2)
  #abline(-mean(merge_data$Valor.x - merge_data$Valor.y, na.rm = T)+10,1, col="red",lwd=2)
  #abline(-mean(merge_data$Valor.x - merge_data$Valor.y, na.rm = T)-10,1, col="red",lwd=2)
  # abline(0,1, col="blue",lwd=2)
  
  #windows()
  plot(merge_data$posixdate,(merge_data$Valor.x - merge_data$Valor.y), pch=".")
  #points(ind$posixdate,ind$index*10, pch=19, col=2, cex=1)
  #identify(merge_data$posixdate, (merge_data$Valor.x - merge_data$Valor.y),labels = merge_data$posixdate) 
  
  diff_temp <- diff(data_1$Valor)
  diff_time <- diff(data_1$posixdate)
  indic = abs(diff_temp) /as.numeric(diff_time)
  indic[which(indic < 0.3)] <- NA
  
  #windows()
  plot(data_1$posixdate[-1],indic, ylim=c(0,max(indic, na.rm=T)), pch=19, xlab="Time", ylab="Temperature Index", main="Station 1: Temperature Quality Index", sub="Index = abs(diff_temp) / diff_time")
  abline(h=0.3, col=2, lwd=2)
  #identify(data_1$posixdate[-1], indic,labels = data_1$posixdate[-1]) 
  
  diff_temp <- diff(data_2$Valor)
  diff_time <- diff(data_2$posixdate)
  indic = abs(diff_temp) /as.numeric(diff_time)
  indic[which(indic < 0.3)] <- NA
  ind <- data.frame(posixdate=data_2$posixdate[-1], index = indic)
  
  #windows()
  plot(data_2$posixdate[-1],indic, ylim=c(0,max(indic, na.rm=T)), pch=19, xlab="Time", ylab="Temperature Index", main="Station 2: Temperature Quality Index", sub="Index = abs(diff_temp) / diff_time")
  abline(h=0.3, col=2, lwd=2)
  #identify(data_2$posixdate[-1], indic,labels = data_2$posixdate[-1]) 
  
    dev.off()
}

temp_func(P032, P102)


### Temperature: Cuashi Data####

source("U:/HiWi_ER/CUAHSI/f_write4cuahsi.R")

load("R_Data/A145B01TEMPE.RData") ; data <- temp ; statname = "Eriste"

load("R_Data/A258B01TEMPE.RData") ; data <- temp ; statname = "Campo"

load("R_Data/EM47G02TEMPE.RData") ; data <- temp ; statname = "El_Grado"

load("R_Data/EM48B02TEMPE.RData") ; data <- temp ; statname = "Barasona"

load("R_Data/P029G01TEMPE.RData") ; data <- temp ; statname = "Samper"

load("R_Data/P030B01TEMPE.RData") ; data <- temp ; statname = "Las_Paules"

load("R_Data/P032S17TEMPE.RData") ; data <- temp ; statname = "Castigaleu"

load("R_Data/P102Z02TEMPE.RData") ; data <- temp ; statname = "Cerler"

# Select time frame for CUASHI upload
colnames(data) = c("Tag", "date", "DataValue")
data <- subset(data, format(date, "%Y") >"2010")

write4cuahsi(cdata=data, dest_file=paste(statname, "_temp", sep=""), sitecode=statname, varcode="temperature_15" , methodc="temperature_SAIH" , sourcec="SAIH", censorc="nc", OffsetValue="", qualityc=1, ValueAccuracy="", maxrec=700000)


### Temperature: Final check####

  
final <- function(filename){
  
cuashi_file = paste("CuashiData/", filename, sep="")
cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )

cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))

sub2008 <- subset(cuashi, format(LocalDateTime, "%Y")=="2008")
sub2009 <- subset(cuashi, format(LocalDateTime, "%Y")=="2009")
sub2010 <- subset(cuashi, format(LocalDateTime, "%Y")=="2010")
sub2011 <- subset(cuashi, format(LocalDateTime, "%Y")=="2011")
sub2012 <- subset(cuashi, format(LocalDateTime, "%Y")=="2012")
sub2013 <- subset(cuashi, format(LocalDateTime, "%Y")=="2013")
sub2014 <- subset(cuashi, format(LocalDateTime, "%Y")=="2014")
sub2015 <- subset(cuashi, format(LocalDateTime, "%Y")=="2015")
sub2016 <- subset(cuashi, format(LocalDateTime, "%Y")=="2016")

pdf(paste("plots/",filename,"_Cuashi",".pdf",sep = ""),width=10,height=8)

if(nrow(cuashi) >0){
  plot(cuashi$LocalDateTime, cuashi$DataValue, main=paste(filename), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(cuashi$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2008) >0){
  plot(sub2008$LocalDateTime, sub2008$DataValue, main=paste(filename,"_2008",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2008$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2009) >0){
  plot(sub2009$LocalDateTime, sub2009$DataValue, main=paste(filename,"_2009",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2009$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2010) >0){
  plot(sub2010$LocalDateTime, sub2010$DataValue, main=paste(filename,"_2010",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2010$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2011) >0){
  plot(sub2011$LocalDateTime, sub2011$DataValue, main=paste(filename,"_2011",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2011$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2012) >0){
  plot(sub2012$LocalDateTime, sub2012$DataValue, main=paste(filename,"_2012",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2012$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2013) >0){
  plot(sub2013$LocalDateTime, sub2013$DataValue, main=paste(filename,"_2013",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2013$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2014) >0){
  plot(sub2014$LocalDateTime, sub2014$DataValue, main=paste(filename,"_2014",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2014$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2015) >0){
  plot(sub2015$LocalDateTime, sub2015$DataValue, main=paste(filename,"_2015",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2015$DataValue, na.rm=T), digits = 1))))}

if(nrow(sub2016) >0){
  plot(sub2016$LocalDateTime, sub2016$DataValue, main=paste(filename,"_2016",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2016$DataValue, na.rm=T), digits = 1))))}

dev.off()

}

dir(path="CuashiData")

final("P102Z02TEMPE.csv")




### Precipitation: Data reading and processing ####


precip_saih <-  function(single_file){

precip <- read.table(single_file , stringsAsFactors = F, sep=";", dec= ",", skip = 1, na.strings = "NO DATO") 
colnames(precip) <- c("Tag","Fecha","Valor")

precip$date      = as.POSIXct(strptime(precip$Fecha, format="%d/%m/%y %H:%M", tz="etc/GMT-1"))

precip$date_CET  = as.POSIXct(strptime(precip$Fecha, format="%d/%m/%y %H:%M", tz="CET"))

#from 2013 on switch to summer time in data
precip$date[which(format(precip$date_CET, "%Y") > "2012")] <-  precip$date_CET[which(format(precip$date_CET, "%Y") > "2012")]
#date automatically adjusted and converted to GMT+1

sorted_ix=sort.int(precip$date, index.return = TRUE)$ix
precip = precip[sorted_ix,]

precip <- data.frame(tag=precip$Tag, date=precip$date, x=precip$Valor)

filename <- precip[1,1]

#aggregate to full hour for station comparison SESAM
precip$date_full_hour =as.POSIXct(trunc(precip$date, unit="hour"))
na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
precip_aggr <-  aggregate(x = precip$x, by=list(date=precip$date_full_hour), FUN=na_sum)
precip$date_full_hour = NULL

#Save as .RData
save(file = paste0("R_Data/",filename,".RData"), list="precip")  
save(file = paste0("R_Data/",filename,"_aggr.RData"), list="precip_aggr") 

sub2006 <- subset(precip, format(date, "%Y")=="2006")
sub2007 <- subset(precip, format(date, "%Y")=="2007")
sub2008 <- subset(precip, format(date, "%Y")=="2008")
sub2009 <- subset(precip, format(date, "%Y")=="2009")
sub2010 <- subset(precip, format(date, "%Y")=="2010")
sub2011 <- subset(precip, format(date, "%Y")=="2011")
sub2012 <- subset(precip, format(date, "%Y")=="2012")
sub2013 <- subset(precip, format(date, "%Y")=="2013")
sub2014 <- subset(precip, format(date, "%Y")=="2014")
sub2015 <- subset(precip, format(date, "%Y")=="2015")
sub2016 <- subset(precip, format(date, "%Y")=="2016")

pdf(paste("plots/",filename,".pdf",sep = ""),width=10,height=8)

if(nrow(precip) >0){
  plot(precip$date, precip$x, main=paste(filename), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(precip$x, na.rm=T), digits = 1))))}

if(nrow(sub2006) >0){
  plot(sub2006$date, sub2006$x, main=paste(filename,"_2006",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2006$x, na.rm=T), digits = 1))))}

if(nrow(sub2007) >0){
  plot(sub2007$date, sub2007$x, main=paste(filename,"_2007",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2007$x, na.rm=T), digits = 1))))}

if(nrow(sub2008) >0){
  plot(sub2008$date, sub2008$x, main=paste(filename,"_2008",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2008$x, na.rm=T), digits = 1))))}

if(nrow(sub2009) >0){
  plot(sub2009$date, sub2009$x, main=paste(filename,"_2009",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2009$x, na.rm=T), digits = 1))))}

if(nrow(sub2010) >0){
  plot(sub2010$date, sub2010$x, main=paste(filename,"_2010",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2010$x, na.rm=T), digits = 1))))}

if(nrow(sub2011) >0){
  plot(sub2011$date, sub2011$x, main=paste(filename,"_2011",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}

if(nrow(sub2012) >0){
  plot(sub2012$date, sub2012$x, main=paste(filename,"_2012",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2012$x, na.rm=T), digits = 1))))}

if(nrow(sub2013) >0){
  plot(sub2013$date, sub2013$x, main=paste(filename,"_2013",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2013$x, na.rm=T), digits = 1))))}

if(nrow(sub2014) >0){
  plot(sub2014$date, sub2014$x, main=paste(filename,"_2014",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2014$x, na.rm=T), digits = 1))))}

if(nrow(sub2015) >0){
  plot(sub2015$date, sub2015$x, main=paste(filename,"_2015",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2015$x, na.rm=T), digits = 1))))}

if(nrow(sub2016) >0){
  plot(sub2016$date, sub2016$x, main=paste(filename,"_2016",sep=""), ylab = "Precipitation [mm/15min]", xlab= "Time", type="l")
  legend("topleft", c("Range", as.character(round(range(sub2016$x, na.rm=T), digits = 1))))}

jump <- data.frame(2006:2016,rep(NA,11))
if(length(precip$date[which(precip$date == "2006-03-26 01:45:00")]) > 0){
  jump[1,2] <- precip$date[which(precip$date == "2006-03-26 01:45:00")+1] - precip$date[which(precip$date == "2006-03-26 01:45:00")]}
if(length(precip$date[which(precip$date == "2007-03-25 01:45:00")]) > 0){
  jump[2,2] <- precip$date[which(precip$date == "2007-03-25 01:45:00")+1] - precip$date[which(precip$date == "2007-03-25 01:45:00")]}
if(length(precip$date[which(precip$date == "2008-03-30 01:45:00")]) > 0){
  jump[3,2] <- precip$date[which(precip$date == "2008-03-30 01:45:00")+1] - precip$date[which(precip$date == "2008-03-30 01:45:00")]}
if(length(precip$date[which(precip$date == "2009-03-29 01:45:00")]) > 0){
  jump[4,2] <- precip$date[which(precip$date == "2009-03-29 01:45:00")+1] - precip$date[which(precip$date == "2009-03-29 01:45:00")]}
if(length(precip$date[which(precip$date == "2010-03-28 01:45:00")]) > 0){
  jump[5,2] <- precip$date[which(precip$date == "2010-03-28 01:45:00")+1] - precip$date[which(precip$date == "2010-03-28 01:45:00")]}
if(length(precip$date[which(precip$date == "2011-03-27 01:45:00")]) > 0){
  jump[6,2] <- precip$date[which(precip$date == "2011-03-27 01:45:00")+1] - precip$date[which(precip$date == "2011-03-27 01:45:00")]}
if(length(precip$date[which(precip$date == "2012-03-25 01:45:00")]) > 0){
  jump[7,2] <- precip$date[which(precip$date == "2012-03-25 01:45:00")+1] - precip$date[which(precip$date == "2012-03-25 01:45:00")]}
if(length(precip$date[which(precip$date == "2013-03-31 01:45:00")]) > 0){
  jump[8,2] <- precip$date[which(precip$date == "2013-03-31 01:45:00")+1] - precip$date[which(precip$date == "2013-03-31 01:45:00")]}
if(length(precip$date[which(precip$date == "2014-03-30 01:45:00")]) > 0){
  jump[9,2] <- precip$date[which(precip$date == "2014-03-30 01:45:00")+1] - precip$date[which(precip$date == "2014-03-30 01:45:00")]}
if(length(precip$date[which(precip$date == "2015-03-29 01:45:00")]) > 0){
  jump[10,2] <- precip$date[which(precip$date == "2015-03-29 01:45:00")+1] - precip$date[which(precip$date == "2015-03-29 01:45:00")]}
if(length(precip$date[which(precip$date == "2016-03-27 01:45:00")]) > 0){
  jump[11,2] <- precip$date[which(precip$date == "2016-03-27 01:45:00")+1] - precip$date[which(precip$date == "2016-03-27 01:45:00")]}

plot(jump, pch=19,main=paste("Winter time to Summer time ",filename,sep = ""), xlab="Year", ylab="Time between 01:45 and next time step")

#Dublicate check
if(anyDuplicated(precip$date)>0){duplicates_file = (paste("Dublicates in ", single_file))
} else {duplicates_file = paste("No dublicates in ", single_file)}

if(nrow(precip) >0){
  plot(precip$date, precip$x, main=paste(filename),ylab="", xlab="", axes=F, type="n")
  legend("center", paste(duplicates_file), bg="grey90")}

#Time difference
diff_time <- diff(precip$date)
diff_time[which(as.numeric(diff_time) == 15)] <- NA
plot(precip$date[1:(length(precip$date)-1)],diff_time, ylim=c(0,max(as.numeric(diff_time), na.rm=T)),pch=19, xlab="Date", ylab="Time difference [min]", main="Time Difference not 15min")

dev.off()

}

dir()
single_file <- "R065_B85_prec_2016_12_01.txt"

precip_saih(single_file)


### Precipitation: Quality control ####

load("R_Data/A047B85PQUIN_aggr.RData") ; A047_aggr <- precip_aggr
load("R_Data/P030B85PQUIN_aggr.RData") ; P030_aggr <- precip_aggr
load("R_Data/P031S85PQUIN_aggr.RData") ; P031_aggr <- precip_aggr
load("R_Data/P032S85PQUIN_aggr.RData") ; P032_aggr <- precip_aggr
load("R_Data/R063B85PQUIN_aggr.RData") ; R063_aggr <- precip_aggr
load("R_Data/R065B85PQUIN_aggr.RData") ; R065_aggr <- precip_aggr

#SESAM stations
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Via1_merged_aggr.RData") ; Via1_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Via2_merged_aggr.RData") ; Via2_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Cab1_merged_aggr.RData") ; Cab1_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Cab2_merged_aggr.RData") ; Cab2_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Ceg1_merged_aggr.RData") ; Ceg1_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Ceg2_merged_aggr.RData") ; Ceg2_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Las1_merged_aggr.RData") ; Las1_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Las2_merged_aggr.RData") ; Las2_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Lascuarre_tube_merged_aggr.RData") ; Lascuarre_tube_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUASHI/SESAM/MergeData/Car_merged_aggr.RData")  ; Car_aggr  <- cont_collected_aggr


#Function just using prior aggregated data
station_compare_aggr <- function(data_3, data_4){
  
  pdf(paste("Plots/","station_compare_prec.pdf",sep = ""),width=10,height=8)
  
  nas_3 = which(is.na(data_3$x))
  nas_4 = which(is.na(data_4$x))
  if(length(nas_3) > 0) data_3 = data_3[-which(is.na(data_3$x)),] #station comparison just when data available
  if(length(nas_4) > 0) data_4 = data_4[-which(is.na(data_4$x)),] #station comparison just when data available
                                  
  # Cumsum-Plot station 1
  plot(data_3$date, cumsum(data_3$x), pch=".", main="Station 1",sub="hourly, continous times series",xlab="Time", ylab="Cumulative Sum")
  
  # Cumsum-Plot station 1
  plot(data_4$date, cumsum(data_4$x), pch=".", main="Station 2",sub="hourly, continous times series",xlab="Time", ylab="Cumulative Sum")
  
  # Cumsum-Comparison-Plot hourly continous data
  merge_data = merge(data_3, data_4, by = "date", all = FALSE) 
  merge_data$x.x[which(is.na(merge_data$x.x))] = 0 # for comparative cunsum plot NAs to zero
  merge_data$x.y[which(is.na(merge_data$x.y))] = 0
  
  plot(cumsum(merge_data$x.x), cumsum(merge_data$x.y), pch=".",  main= "Station comparison using cumulative sum",sub="hourly,continous times series; merge (... all=FALSE);", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  abline(0,1, col="orange") 
  
  # Individual stations hourly continours data, merge()
  plot(merge_data$date, cumsum(merge_data$x.x), pch=".", ylim=c(0,max(c(cumsum(merge_data$x.x),cumsum(merge_data$x.y)))),main="Station comparison",sub="hourly, continous times series; merge (... all=FALSE)",xlab="Time", ylab="Cumulative Sum")
  points(merge_data$date, cumsum(merge_data$x.y), pch=".", col="red")
  points(merge_data$date, abs((cumsum(merge_data$x.x)-cumsum(merge_data$x.y))), pch=".", col="blue")
  legend("topleft",c("Station 1","Station 2","|Station 1 - Station 2|"), pch=19, col=c("black","red","blue"))
  
  dev.off()
}

station_compare_aggr(R063_aggr,Cab1_aggr)


### Precipitation: Cuashi Data####

load("R_Data/A047B85PQUIN.RData") ; data <- precip ; statname <- "Capella"

load("R_Data/P030B85PQUIN.RData") ; data <- precip ; statname <- "Las_Paules"

load("R_Data/P031S85PQUIN.RData") ; data <- precip ; statname <- "Casa_Llera"

load("R_Data/P032S85PQUIN.RData") ; data <- precip ; statname <- "Castigaleu"

load("R_Data/R063B85PQUIN.RData") ; data <- precip ; statname <- "Chordal"

load("R_Data/R065B85PQUIN.RData") ; data <- precip ; statname <- "Calvera"

# Select time frame for CUASHI upload

source("U:/HiWi_ER/CUAHSI/f_write4cuahsi.R")

data <- subset(data, format(date, "%Y") >"2010")
colnames(data) = c("Tag","date", "DataValue")


write4cuahsi(cdata=data, dest_file=paste(statname,"_Precip", sep=""), sitecode=statname, varcode="rainfall_15" , methodc="tipping_bucket_SAIH" , sourcec="SAIH", censorc="nc", OffsetValue="", qualityc=1, ValueAccuracy="", maxrec=700000)


### Precipitation: Final Check ####

final <- function(filename){
  
  cuashi_file = paste("CuashiData/", filename, sep="")
  cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )
  
  cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))
  
  sub2011 <- subset(cuashi, format(LocalDateTime, "%Y")=="2011")
  sub2012 <- subset(cuashi, format(LocalDateTime, "%Y")=="2012")
  sub2013 <- subset(cuashi, format(LocalDateTime, "%Y")=="2013")
  sub2014 <- subset(cuashi, format(LocalDateTime, "%Y")=="2014")
  sub2015 <- subset(cuashi, format(LocalDateTime, "%Y")=="2015")
  sub2016 <- subset(cuashi, format(LocalDateTime, "%Y")=="2016")
  #filename <- cuashi$SiteCode[1]
  
  pdf(paste("Plots/",filename,"_Cuashi.pdf",sep = ""),width=10,height=8)
  
  if(nrow(cuashi) >0){
    plot(cuashi$LocalDateTime, cuashi$DataValue, main=paste(filename), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    #abline(v=as.numeric(cont_aggr$date[which(is.na(cont_aggr$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(cuashi$DataValue, na.rm=T), digits = 1))))}
  
  cont_cunsum <- cuashi[-which(is.na(cuashi$DataValue)),]
  plot(cont_cunsum$LocalDateTime, cumsum(cont_cunsum$DataValue), pch=".",xlab="Time",ylab="Cumulative Sum")
  
  if(nrow(sub2011) >0){
    plot(sub2011$LocalDateTime, sub2011$DataValue, main=paste(filename,"_2011",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    #abline(v=as.numeric(sub2011$date[which(is.na(sub2011$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(sub2011$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$LocalDateTime, sub2012$DataValue, main=paste(filename,"_2012",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2012$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$LocalDateTime, sub2013$DataValue, main=paste(filename,"_2013",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2013$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$LocalDateTime, sub2014$DataValue, main=paste(filename,"_2014",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2014$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$LocalDateTime, sub2015$DataValue, main=paste(filename,"_2015",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2015$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$LocalDateTime, sub2016$DataValue, main=paste(filename,"_2016",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2016$DataValue, na.rm=T), digits = 1))))}
  
  dev.off()
  
}

dir(path="CuahsiData")
final("R065B85PQUIN.csv")

dir()



### Humidity: Data reading and processing####

humi_saih <- function(single_file){
  
  humi <- read.table(single_file , stringsAsFactors = F, sep=";", dec= ",", skip = 1, na.strings = c("NO DATO")) ;
  colnames(humi) <- c("Tag","Fecha","Valor")
  
  humi$posixdate = as.POSIXct(strptime(humi$Fecha, format="%d/%m/%y %H:%M", tz="etc/GMT-1"))
  humi$posixdate_CET  = as.POSIXct(strptime(humi$Fecha, format="%d/%m/%y %H:%M", tz="CET"))
  
  #from 2013 on switch to summer time in data
  humi$posixdate[which(format(humi$posixdate_CET, "%Y") > "2012")] <-  humi$posixdate_CET[which(format(humi$posixdate_CET, "%Y") > "2012")]
  #date automatically adjusted and converted to GMT+1
  
  sorted_ix=sort.int(humi$posixdate, index.return = TRUE)$ix
  humi = humi[sorted_ix,]
  
  filename <- humi[1,1]
  
  # Time frame for CUASHI upload 01.01.2011 - end data (01.12.2016)
  humi <- subset(humi, format(posixdate, "%Y") >"2010")
  
  humi <- data.frame(Tag=humi$Tag, posixdate=humi$posixdate, Valor=humi$Valor)
  
  save(file = paste0("R_Data/",filename,".RData"), list="humi")
  
  sub2006 <- subset(humi, format(posixdate, "%Y")=="2006")
  sub2007 <- subset(humi, format(posixdate, "%Y")=="2007")
  sub2008 <- subset(humi, format(posixdate, "%Y")=="2008")
  sub2009 <- subset(humi, format(posixdate, "%Y")=="2009")
  sub2010 <- subset(humi, format(posixdate, "%Y")=="2010")
  sub2011 <- subset(humi, format(posixdate, "%Y")=="2011")
  sub2012 <- subset(humi, format(posixdate, "%Y")=="2012")
  sub2013 <- subset(humi, format(posixdate, "%Y")=="2013")
  sub2014 <- subset(humi, format(posixdate, "%Y")=="2014")
  sub2015 <- subset(humi, format(posixdate, "%Y")=="2015")
  sub2016 <- subset(humi, format(posixdate, "%Y")=="2016")
  
  pdf(paste("plots/",filename,".pdf",sep = ""),width=10,height=8)
  
  if(nrow(humi) >0){
    plot(humi$posixdate, humi$Valor, main=paste(filename), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(humi$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2006) >0){
    plot(sub2006$posixdate, sub2006$Valor, main=paste(filename,"_2006",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2006$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2007) >0){
    plot(sub2007$posixdate, sub2007$Valor, main=paste(filename,"_2007",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2007$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2008) >0){
    plot(sub2008$posixdate, sub2008$Valor, main=paste(filename,"_2008",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2008$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2009) >0){
    plot(sub2009$posixdate, sub2009$Valor, main=paste(filename,"_2009",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2009$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2010) >0){
    plot(sub2010$posixdate, sub2010$Valor, main=paste(filename,"_2010",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2010$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$posixdate, sub2011$Valor, main=paste(filename,"_2011",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2011$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$posixdate, sub2012$Valor, main=paste(filename,"_2012",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2012$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$posixdate, sub2013$Valor, main=paste(filename,"_2013",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2013$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$posixdate, sub2014$Valor, main=paste(filename,"_2014",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2014$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$posixdate, sub2015$Valor, main=paste(filename,"_2015",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2015$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$posixdate, sub2016$Valor, main=paste(filename,"_2016",sep=""), ylab = "Humidity %", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2016$Valor, na.rm=T), digits = 1))))}
  
  jump <- data.frame(2006:2016,rep(NA,11))
  if(length(humi$posixdate[which(humi$posixdate == "2006-03-26 01:45:00")]) > 0){
    jump[1,2] <- humi$posixdate[which(humi$posixdate == "2006-03-26 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2006-03-26 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2007-03-25 01:45:00")]) > 0){
    jump[2,2] <- humi$posixdate[which(humi$posixdate == "2007-03-25 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2007-03-25 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2008-03-30 01:45:00")]) > 0){
    jump[3,2] <- humi$posixdate[which(humi$posixdate == "2008-03-30 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2008-03-30 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2009-03-29 01:45:00")]) > 0){
    jump[4,2] <- humi$posixdate[which(humi$posixdate == "2009-03-29 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2009-03-29 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2010-03-28 01:45:00")]) > 0){
    jump[5,2] <- humi$posixdate[which(humi$posixdate == "2010-03-28 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2010-03-28 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2011-03-27 01:45:00")]) > 0){
    jump[6,2] <- humi$posixdate[which(humi$posixdate == "2011-03-27 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2011-03-27 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2012-03-25 01:45:00")]) > 0){
    jump[7,2] <- humi$posixdate[which(humi$posixdate == "2012-03-25 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2012-03-25 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2013-03-31 01:45:00")]) > 0){
    jump[8,2] <- humi$posixdate[which(humi$posixdate == "2013-03-31 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2013-03-31 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2014-03-30 01:45:00")]) > 0){
    jump[9,2] <- humi$posixdate[which(humi$posixdate == "2014-03-30 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2014-03-30 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2015-03-29 01:45:00")]) > 0){
    jump[10,2] <- humi$posixdate[which(humi$posixdate == "2015-03-29 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2015-03-29 01:45:00")]}
  if(length(humi$posixdate[which(humi$posixdate == "2016-03-27 01:45:00")]) > 0){
    jump[11,2] <- humi$posixdate[which(humi$posixdate == "2016-03-27 01:45:00")+1] - humi$posixdate[which(humi$posixdate == "2016-03-27 01:45:00")]}
  
  plot(jump, pch=19,main=paste("Winter time to Summer time ",filename,sep = ""), xlab="Year", ylab="Time between 01:45 and next time step")
  
  #Dublicate check
  if(anyDuplicated(humi$posixdate)>0){duplicates_file = (paste("Dublicates in ", single_file))
  } else {duplicates_file = paste("No dublicates in ", single_file)}
  
  if(nrow(humi) >0){
    plot(humi$posixdate, humi$Valor, main=paste(filename),ylab="", xlab="", axes=F, type="n")
    legend("center", paste(duplicates_file), bg="grey90")}
  
  #Time difference
  diff_time <- diff(humi$posixdate)
  diff_time[which(as.numeric(diff_time) == 15)] <- NA
  plot(humi$posixdate[1:(length(humi$posixdate)-1)],diff_time, ylim=c(0,max(as.numeric(diff_time), na.rm=T)),pch=19, xlab="Date", ylab="Time difference [min]", main="Time Difference not 15min")
  
  dev.off()
  
}

dir()
single_file <- "EM48_B03_humid_2016_12_01.txt"

humi_saih(single_file)


### Humidity: Quality control####

load("R_Data/EM47G03HUMED.RData") ; EM47 <- humi
load("R_Data/EM48B03HUMED.RData") ; EM48 <- humi


humi_func <- function(data_1, data_2){
  
  pdf(paste("plots/station_compare_humi.pdf",sep = ""),width=10,height=8)
  
  diff_humi <- diff(data_1$Valor)
  diff_time <- diff(data_1$posixdate)
  indic = abs(diff_humi) /as.numeric(diff_time)
  indic[which(indic < 0.5)] <- NA
  
  #windows()
  plot(data_1$posixdate[-1],indic, ylim=c(0,max(indic, na.rm=T)), pch=19, xlab="Time", ylab="Humidity Index", main="Station 1: Humidity Quality Index", sub="Index = abs(diff_humi) / diff_time")
  abline(h=0.5, col=2, lwd=2)
  #identify(data_1$posixdate[-1], indic,labels = data_1$posixdate[-1]) 
  
  diff_humi <- diff(data_2$Valor)
  diff_time <- diff(data_2$posixdate)
  indic = abs(diff_humi) /as.numeric(diff_time)
  indic[which(indic < 0.5)] <- NA
  
  #windows()
  plot(data_2$posixdate[-1],indic, ylim=c(0,max(indic, na.rm=T)), pch=19, xlab="Time", ylab="Humidity Index", main="Station 2: Humidity Quality Index", sub="Index = abs(diff_humi) / diff_time")
  abline(h=0.5, col=2, lwd=2)
  #identify(data_1$posixdate[-1], indic,labels = data_1$posixdate[-1]) 
  
  merge_data = merge(data_1, data_2, by = "posixdate", all = FALSE) 
  
  hist(merge_data$Valor.x - merge_data$Valor.y, col=2, breaks=50)
  
  plot(merge_data$Valor.x, merge_data$Valor.y, pch=19, xlab = "data_1", ylab = "data_2", main = "Station comparison")
  
  plot(merge_data$posixdate,(merge_data$Valor.x - merge_data$Valor.y), pch=".")
  #points(ind$posixdate,ind$index*10, pch=19, col=2, cex=1)
  
  dev.off()
}

humi_func(EM47, EM48)


### Humidity: Cuashi Data####

load("R_Data/EM47G03HUMED.RData") ; joined <- humi ; statname <- "El_Grado"

load("R_Data/EM48B03HUMED.RData") ; joined <- humi ; statname <- "Barasona"

# Select time frame for CUASHI upload
joined <- subset(joined, format(date, "%Y") >"2010")

n= nrow(joined)

dtutc<- format(joined$posixdate, tz="UTC")
dtloc<- format(joined$posixdate, tz="Etc/GMT-1") # LocalDateTime winter time GMT+1
utc_offset = as.numeric(format(joined$posixdate, format = "%z"))/100
sitecode <- rep(statname,n)
varcode <- rep("relative_humidity",n)
censorc	<- rep("nc", n) 
methodc	<- rep("relative_humidity_probe", n)
sourcec <- rep("SAIH", n)
qualityc <- rep("1",n)

output_table=data.frame(DataValue=round(joined$Valor, digits = 3),
                        ValueAccuracy="",
                        LocalDateTime=dtloc,
                        UTCOffset=utc_offset,
                        DateTimeUTC=dtutc,
                        SiteCode=sitecode,
                        VariableCode=varcode,
                        OffsetValue="",
                        OffsetTypeCode="",
                        CensorCode = censorc,
                        QualifierCode="",
                        MethodCode = methodc,
                        SourceCode = sourcec,
                        SampleCode="",
                        DerivedFromID="",
                        QualityControlLevelCode= qualityc )

write.table(file=paste0("CuahsiData/",statname, "_humid.csv"),x = output_table, na ="-9999", sep=",", row.names = FALSE, quote = FALSE)

#paste0("CuashiData/",station_id_golm, "Hobo.csv")


### Humidity: Final Check ####

final <- function(filename){
  
  cuashi_file = paste("CuahsiData/", filename, sep="")
  cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )
  
  cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))
  
  sub2011 <- subset(cuashi, format(LocalDateTime, "%Y")=="2011")
  sub2012 <- subset(cuashi, format(LocalDateTime, "%Y")=="2012")
  sub2013 <- subset(cuashi, format(LocalDateTime, "%Y")=="2013")
  sub2014 <- subset(cuashi, format(LocalDateTime, "%Y")=="2014")
  sub2015 <- subset(cuashi, format(LocalDateTime, "%Y")=="2015")
  sub2016 <- subset(cuashi, format(LocalDateTime, "%Y")=="2016")
  #filename <- cuashi$SiteCode[1]
  
  #plot for 'problem report'
  jpeg("Plots/EM48.jpeg")
  plot(cuashi$LocalDateTime, cuashi$DataValue, ylim=c(0,100),main=paste(filename), ylab = "Humidity [%]", xlab= "Time", type="l")
  abline(h=100,lty="dotted")
  dev.off()
  
  savePlot(filename="Plots/EM47.jpg", type="jpeg")
  
  plot()
  
  pdf(paste("Plots/",filename,"_Cuashi.pdf",sep = ""),width=10,height=8)
  
  if(nrow(cuashi) >0){
    plot(cuashi$LocalDateTime, cuashi$DataValue, main=paste(filename), ylab = "Humidity [%]", xlab= "Time", type="l")
    #abline(v=as.numeric(cont_aggr$date[which(is.na(cont_aggr$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(cuashi$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$LocalDateTime, sub2011$DataValue, main=paste(filename,"_2011",sep=""), ylab = "Humidity [%]", xlab= "Time", type="l")
    #abline(v=as.numeric(sub2011$date[which(is.na(sub2011$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(sub2011$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$LocalDateTime, sub2012$DataValue, main=paste(filename,"_2012",sep=""), ylab = "Humidity [%]", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2012$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$LocalDateTime, sub2013$DataValue, main=paste(filename,"_2013",sep=""), ylab = "Humidity [%]", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2013$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$LocalDateTime, sub2014$DataValue, main=paste(filename,"_2014",sep=""), ylab = "Humidity [%]", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2014$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$LocalDateTime, sub2015$DataValue, main=paste(filename,"_2015",sep=""), ylab = "Humidity [%]", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2015$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$LocalDateTime, sub2016$DataValue, main=paste(filename,"_2016",sep=""), ylab = "Humidity [%]", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2016$DataValue, na.rm=T), digits = 1))))}
  
  dev.off()
  
}

dir(path="CuahsiData")
final("EM47G03HUMED.csv")



### Radiation: Data reading and processing####

radi_saih <- function(single_file){
  
  radi <- read.table(single_file , stringsAsFactors = F, sep=";", dec= ",", skip = 1, na.strings = c("NO DATO")) ;
  colnames(radi) <- c("Tag","Fecha","Valor")
  
  radi$posixdate = as.POSIXct(strptime(radi$Fecha, format="%d/%m/%y %H:%M", tz="etc/GMT-1"))
  radi$posixdate_CET  = as.POSIXct(strptime(radi$Fecha, format="%d/%m/%y %H:%M", tz="CET"))
  
  #from 2013 on switch to summer time in data
  radi$posixdate[which(format(radi$posixdate_CET, "%Y") > "2012")] <-  radi$posixdate_CET[which(format(radi$posixdate_CET, "%Y") > "2012")]
  #date automatically adjusted and converted to GMT+1
  
  sorted_ix=sort.int(radi$posixdate, index.return = TRUE)$ix
  radi = radi[sorted_ix,]
  
  #EM47 Problem sudden reduction values from 25th of August on (SAIH: sensor did not work well)

  #plot(sub2016$posixdate[22500:23100], sub2016$Valor[22500:23100], main=paste(filename,"_2016",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
  if(single_file=="EM47_G04_radia_2016_12_01.txt"){
  radi$Valor[which(radi$posixdate > "2016-08-25 11:45:00")] <-  NA
  }
  
  plot(sub2016$posixdate[22420:22450], sub2016$Valor[22420:22450], main=paste(filename,"_2016 21.8",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
  
  filename <- radi[1,1]
  
  radi <- data.frame(Tag=radi$Tag, posixdate=radi$posixdate, Valor=radi$Valor)
  
  save(file = paste0("R_Data/",filename,".RData"), list="radi")
  
  sub2006 <- subset(radi, format(posixdate, "%Y")=="2006")
  sub2007 <- subset(radi, format(posixdate, "%Y")=="2007")
  sub2008 <- subset(radi, format(posixdate, "%Y")=="2008")
  sub2009 <- subset(radi, format(posixdate, "%Y")=="2009")
  sub2010 <- subset(radi, format(posixdate, "%Y")=="2010")
  sub2011 <- subset(radi, format(posixdate, "%Y")=="2011")
  sub2012 <- subset(radi, format(posixdate, "%Y")=="2012")
  sub2013 <- subset(radi, format(posixdate, "%Y")=="2013")
  sub2014 <- subset(radi, format(posixdate, "%Y")=="2014")
  sub2015 <- subset(radi, format(posixdate, "%Y")=="2015")
  sub2016 <- subset(radi, format(posixdate, "%Y")=="2016")
  
  pdf(paste("plots/",filename,".pdf",sep = ""),width=10,height=8)
  
  if(nrow(radi) >0){
    plot(radi$posixdate, radi$Valor, main=paste(filename), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(radi$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2006) >0){
    plot(sub2006$posixdate, sub2006$Valor, main=paste(filename,"_2006",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2006$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2007) >0){
    plot(sub2007$posixdate, sub2007$Valor, main=paste(filename,"_2007",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2007$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2008) >0){
    plot(sub2008$posixdate, sub2008$Valor, main=paste(filename,"_2008",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2008$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2009) >0){
    plot(sub2009$posixdate, sub2009$Valor, main=paste(filename,"_2009",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2009$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2010) >0){
    plot(sub2010$posixdate, sub2010$Valor, main=paste(filename,"_2010",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2010$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$posixdate, sub2011$Valor, main=paste(filename,"_2011",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2011$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$posixdate, sub2012$Valor, main=paste(filename,"_2012",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2012$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$posixdate, sub2013$Valor, main=paste(filename,"_2013",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2013$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$posixdate, sub2014$Valor, main=paste(filename,"_2014",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2014$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$posixdate, sub2015$Valor, main=paste(filename,"_2015",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2015$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$posixdate, sub2016$Valor, main=paste(filename,"_2016",sep=""), ylab = "Radiation W/m", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2016$Valor, na.rm=T), digits = 1))))}
  
  
  jump <- data.frame(2006:2016,rep(NA,11))
  if(length(radi$posixdate[which(radi$posixdate == "2006-03-26 01:45:00")]) > 0){
    jump[1,2] <- radi$posixdate[which(radi$posixdate == "2006-03-26 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2006-03-26 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2007-03-25 01:45:00")]) > 0){
    jump[2,2] <- radi$posixdate[which(radi$posixdate == "2007-03-25 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2007-03-25 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2008-03-30 01:45:00")]) > 0){
    jump[3,2] <- radi$posixdate[which(radi$posixdate == "2008-03-30 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2008-03-30 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2009-03-29 01:45:00")]) > 0){
    jump[4,2] <- radi$posixdate[which(radi$posixdate == "2009-03-29 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2009-03-29 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2010-03-28 01:45:00")]) > 0){
    jump[5,2] <- radi$posixdate[which(radi$posixdate == "2010-03-28 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2010-03-28 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2011-03-27 01:45:00")]) > 0){
    jump[6,2] <- radi$posixdate[which(radi$posixdate == "2011-03-27 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2011-03-27 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2012-03-25 01:45:00")]) > 0){
    jump[7,2] <- radi$posixdate[which(radi$posixdate == "2012-03-25 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2012-03-25 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2013-03-31 01:45:00")]) > 0){
    jump[8,2] <- radi$posixdate[which(radi$posixdate == "2013-03-31 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2013-03-31 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2014-03-30 01:45:00")]) > 0){
    jump[9,2] <- radi$posixdate[which(radi$posixdate == "2014-03-30 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2014-03-30 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2015-03-29 01:45:00")]) > 0){
    jump[10,2] <- radi$posixdate[which(radi$posixdate == "2015-03-29 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2015-03-29 01:45:00")]}
  if(length(radi$posixdate[which(radi$posixdate == "2016-03-27 01:45:00")]) > 0){
    jump[11,2] <- radi$posixdate[which(radi$posixdate == "2016-03-27 01:45:00")+1] - radi$posixdate[which(radi$posixdate == "2016-03-27 01:45:00")]}
  
  plot(jump, pch=19,main=paste("Winter time to Summer time ",filename,sep = ""), xlab="Year", ylab="Time between 01:45 and next time step")
  
  #Dublicate check
  if(anyDuplicated(radi$posixdate)>0){duplicates_file = (paste("Dublicates in ", single_file))
  } else {duplicates_file = paste("No dublicates in ", single_file)}
  
  if(nrow(radi) >0){
    plot(radi$posixdate, radi$Valor, main=paste(filename),ylab="", xlab="", axes=F, type="n")
    legend("center", paste(duplicates_file), bg="grey90")}
  
  #Time difference
  diff_time <- diff(radi$posixdate)
  diff_time[which(as.numeric(diff_time) == 15)] <- NA
  plot(radi$posixdate[1:(length(radi$posixdate)-1)],diff_time, ylim=c(0,max(as.numeric(diff_time), na.rm=T)),pch=19, xlab="Date", ylab="Time difference [min]", main="Time Difference not 15min")
  
  dev.off()
  
}

dir()
single_file <- "EM47_G04_radia_2016_12_01.txt"

radi_saih(single_file)


### Radiation: Cuashi Data####

load("R_Data/EM47G04RADIA.RData") ; joined <- radi ; statname <- "El_Grado"

load("R_Data/EM48B04RADIA.RData") ; joined <- radi ; statname <- "Barasona"

# Select time frame for CUASHI upload
joined <- subset(joined, format(posixdate, "%Y") >"2010")

n= nrow(joined)

dtutc<- format(joined$posixdate, tz="UTC")
dtloc<- format(joined$posixdate, tz="Etc/GMT-1") # LocalDateTime winter time GMT+1
utc_offset = as.numeric(format(joined$posixdate, format = "%z"))/100
sitecode <- rep(statname,n)
varcode <- rep("global_radiation",n)
censorc	<- rep("nc", n) 
methodc	<- rep("radiation_sensor", n)
sourcec <- rep("SAIH", n)
qualityc <- rep("1",n)

output_table=data.frame(DataValue=round(joined$Valor, digits = 3),
                        ValueAccuracy="",
                        LocalDateTime=dtloc,
                        UTCOffset=utc_offset,
                        DateTimeUTC=dtutc,
                        SiteCode=sitecode,
                        VariableCode=varcode,
                        OffsetValue="",
                        OffsetTypeCode="",
                        CensorCode = censorc,
                        QualifierCode="",
                        MethodCode = methodc,
                        SourceCode = sourcec,
                        SampleCode="",
                        DerivedFromID="",
                        QualityControlLevelCode= qualityc )

tag = joined[1,1]
write.table(file=paste0("CuahsiData/",statname, "_radia.csv"),x = output_table, na="-9999", sep=",", row.names = FALSE, quote = FALSE)


### Radiation: Final Check ####

final <- function(filename){
  
  cuashi_file = paste("CuahsiData/", filename, sep="")
  cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )
  
  cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))
  
  sub2011 <- subset(cuashi, format(LocalDateTime, "%Y")=="2011")
  sub2012 <- subset(cuashi, format(LocalDateTime, "%Y")=="2012")
  sub2013 <- subset(cuashi, format(LocalDateTime, "%Y")=="2013")
  sub2014 <- subset(cuashi, format(LocalDateTime, "%Y")=="2014")
  sub2015 <- subset(cuashi, format(LocalDateTime, "%Y")=="2015")
  sub2016 <- subset(cuashi, format(LocalDateTime, "%Y")=="2016")
  #filename <- cuashi$SiteCode[1]
  
  pdf(paste("Plots/",filename,"_Cuahsi.pdf",sep = ""),width=10,height=8)
  
  if(nrow(cuashi) >0){
    plot(cuashi$LocalDateTime, cuashi$DataValue, main=paste(filename), ylab = "Radiation [W/m]", xlab= "Time", type="h")
    #abline(v=as.numeric(cont_aggr$date[which(is.na(cont_aggr$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(cuashi$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$LocalDateTime, sub2011$DataValue, main=paste(filename,"_2011",sep=""), ylab = "Radiation [W/m]", xlab= "Time", type="h")
    #abline(v=as.numeric(sub2011$date[which(is.na(sub2011$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(sub2011$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$LocalDateTime, sub2012$DataValue, main=paste(filename,"_2012",sep=""), ylab = "Radiation [W/m]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2012$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$LocalDateTime, sub2013$DataValue, main=paste(filename,"_2013",sep=""), ylab = "Radiation [W/m]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2013$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$LocalDateTime, sub2014$DataValue, main=paste(filename,"_2014",sep=""), ylab = "Radiation [W/m]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2014$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$LocalDateTime, sub2015$DataValue, main=paste(filename,"_2015",sep=""), ylab = "Radiation [W/m]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2015$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$LocalDateTime, sub2016$DataValue, main=paste(filename,"_2016",sep=""), ylab = "Radiation [W/m]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2016$DataValue, na.rm=T), digits = 1))))}
  
  dev.off()
  
}

dir(path="CuahsiData")
final("EM47G04RADIA.csv")



### Water level: Data reading and processing####

dir()
single_file <- "A047_B17_water_2016_12_01.txt"


water_saih <- function(single_file){
  
  water <- read.table(single_file , stringsAsFactors = F, sep=";", dec= ",", skip = 1, na.strings = c("NO DATO")) ;
  colnames(water) <- c("Tag","Fecha","Valor")
  
  water$posixdate = as.POSIXct(strptime(water$Fecha, format="%d/%m/%y %H:%M", tz="etc/GMT-1"))
  water$posixdate_CET  = as.POSIXct(strptime(water$Fecha, format="%d/%m/%y %H:%M", tz="CET"))
  
  #from 2016 on switch to summer time in data
  water$posixdate[which(format(water$posixdate_CET, "%Y") > "2015")] <-  water$posixdate_CET[which(format(water$posixdate_CET, "%Y") > "2015")]
  #date automatically adjusted and converted to GMT+1
  
  sorted_ix=sort.int(water$posixdate, index.return = TRUE)$ix
  water = water[sorted_ix,]
  
  filename <- water[1,1]
  
  water <- data.frame(Tag=water$Tag, posixdate=water$posixdate, Valor=water$Valor)
  
  
  #Float gauge stuck, aground sitting on accumulated sediment
 
  water.dif <- diff(water$Valor)
  water.dif[is.na(water.dif)]=0
  
  period_len = 0*water.dif
  
  wl = length(water.dif)
  i = 1
  
  while (i <= wl)
  {
    if (water.dif[i] != 0)
    {  
      period_len[i]=0
      i = i+1
      next
    }
    pl = which(water.dif[i:wl] != 0)[1] - 1
    if (is.na(pl)) pl=wl-i+1
    period_len[i:(i+pl-1)]=pl
    i = i+ pl
  }
  
  float_stuck = period_len >= (4*24*5) & water$Valor[-1] > 0.3 # set criteria for periods: length period and threshold
  float_stuck[wl+1] = NA
  water$float_stuck = float_stuck
  
  
  # detect 'fake' floods; focus on sediment removal; sudden drop in water level
    # criteria drop of >10cm within 15min & not continue to decrease (after 30 min and 1h still negative)
    # 2015 exceptional year as  float a ot on sediment; excluded from this analysis
  
  water.dif[which(water.dif<(-0.1))] # criteria drop of >10cm within 15min & not continue to decrease
  
  floods = water.dif<(-0.1)
  floods[wl+1]=NA
  
  water$floods = floods
  #which(water$floods == T)
  water$floods[which(water$floods == T)[water.dif[which(water$floods ==T)+2]<0 & water.dif[which(water$floods ==T)+4]<0]] <-  FALSE
  water$floods[which(water$floods == T)[format.Date(water$posixdate[which(water$floods ==T)], "%Y")=="2015"]] <-  FALSE
  
               
  #water.dif[which(water.dif<(-0.1))]
  
  #detected fake events to NA (one hour before until one hour after)
  water$Valor[which(water$floods==T)]   <-  NA
  water$Valor[which(water$floods==T)-1] <-  NA
  water$Valor[which(water$floods==T)-2] <-  NA
  water$Valor[which(water$floods==T)-3] <-  NA
  water$Valor[which(water$floods==T)-4] <-  NA
  water$Valor[which(water$floods==T)+1] <-  NA
  water$Valor[which(water$floods==T)+2] <-  NA
  water$Valor[which(water$floods==T)+3] <-  NA
  water$Valor[which(water$floods==T)+4] <-  NA
  
  # Calculate discharge using rating curve (from Jos, see mail 02.01); Q (m3/s) = 33.422h^2.7869
  
  water$discharge = 33.422 * water$Valor^2.7869 
  
  save(file = paste0("R_Data/",filename,".RData"), list="water")
  
  sub2006 <- subset(water, format(posixdate, "%Y")=="2006")
  sub2007 <- subset(water, format(posixdate, "%Y")=="2007")
  sub2008 <- subset(water, format(posixdate, "%Y")=="2008")
  sub2009 <- subset(water, format(posixdate, "%Y")=="2009")
  sub2010 <- subset(water, format(posixdate, "%Y")=="2010")
  sub2011 <- subset(water, format(posixdate, "%Y")=="2011")
  sub2012 <- subset(water, format(posixdate, "%Y")=="2012")
  sub2013 <- subset(water, format(posixdate, "%Y")=="2013")
  sub2014 <- subset(water, format(posixdate, "%Y")=="2014")
  sub2015 <- subset(water, format(posixdate, "%Y")=="2015")
  sub2016 <- subset(water, format(posixdate, "%Y")=="2016")
  
  pdf(paste("plots/",filename,".pdf",sep = ""),width=10,height=8)
  
  if(nrow(water) >0){
    plot(water$posixdate, water$Valor, main=paste(filename), ylab = "Water level", xlab= "Time", type="l")
    points(water$posixdate[which(water$float_stuck == T)], water$Valor[which(water$float_stuck == T)],col=2, pch="*")
    points(water$posixdate[which(water$floods == T)], water$Valor[which(water$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(water$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2006) >0){
    plot(sub2006$posixdate, sub2006$Valor, main=paste(filename,"_2006",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2006$posixdate[which(sub2006$float_stuck == T)], sub2006$Valor[which(sub2006$float_stuck == T)],col=2, pch="*")
    points(sub2006$posixdate[which(sub2006$floods == T)], sub2006$Valor[which(sub2006$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2006$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2007) >0){
    plot(sub2007$posixdate, sub2007$Valor, main=paste(filename,"_2007",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2007$posixdate[which(sub2007$float_stuck == T)], sub2007$Valor[which(sub2007$float_stuck == T)],col=2, pch="*")
    points(sub2007$posixdate[which(sub2007$floods == T)], sub2007$Valor[which(sub2007$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2007$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2008) >0){
    plot(sub2008$posixdate, sub2008$Valor, main=paste(filename,"_2008",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2008$posixdate[which(sub2008$float_stuck == T)], sub2008$Valor[which(sub2008$float_stuck == T)],col=2, pch="*")
    points(sub2008$posixdate[which(sub2008$floods == T)], sub2008$Valor[which(sub2008$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2008$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2009) >0){
    plot(sub2009$posixdate, sub2009$Valor, main=paste(filename,"_2009",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2009$posixdate[which(sub2009$float_stuck == T)], sub2009$Valor[which(sub2009$float_stuck == T)],col=2, pch="*")
    points(sub2009$posixdate[which(sub2009$floods == T)], sub2009$Valor[which(sub2009$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2009$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2010) >0){
    plot(sub2010$posixdate, sub2010$Valor, main=paste(filename,"_2010",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2010$posixdate[which(sub2010$float_stuck == T)], sub2010$Valor[which(sub2010$float_stuck == T)],col=2, pch="*")
    points(sub2010$posixdate[which(sub2010$floods == T)], sub2010$Valor[which(sub2010$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2010$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$posixdate, sub2011$Valor, main=paste(filename,"_2011",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2011$posixdate[which(sub2011$float_stuck == T)], sub2011$Valor[which(sub2011$float_stuck == T)],col=2, pch="*")
    points(sub2011$posixdate[which(sub2011$floods == T)], sub2011$Valor[which(sub2011$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2011$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$posixdate, sub2012$Valor, main=paste(filename,"_2012",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2012$posixdate[which(sub2012$float_stuck == T)], sub2012$Valor[which(sub2012$float_stuck == T)],col=2, pch="*")
    points(sub2012$posixdate[which(sub2012$floods == T)], sub2012$Valor[which(sub2012$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2012$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$posixdate, sub2013$Valor, main=paste(filename,"_2013",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2013$posixdate[which(sub2013$float_stuck == T)], sub2013$Valor[which(sub2013$float_stuck == T)],col=2, pch="*")
    points(sub2013$posixdate[which(sub2013$floods == T)], sub2013$Valor[which(sub2013$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2013$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$posixdate, sub2014$Valor, main=paste(filename,"_2014",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2014$posixdate[which(sub2014$float_stuck == T)], sub2014$Valor[which(sub2014$float_stuck == T)],col=2, pch="*")
    points(sub2014$posixdate[which(sub2014$floods == T)], sub2014$Valor[which(sub2014$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2014$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$posixdate, sub2015$Valor, main=paste(filename,"_2015",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2015$posixdate[which(sub2015$float_stuck == T)], sub2015$Valor[which(sub2015$float_stuck == T)],col=2, pch="*")
    points(sub2015$posixdate[which(sub2015$floods == T)], sub2015$Valor[which(sub2015$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2015$Valor, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$posixdate, sub2016$Valor, main=paste(filename,"_2016",sep=""), ylab = "Water level", xlab= "Time", type="l")
    points(sub2016$posixdate[which(sub2016$float_stuck == T)], sub2016$Valor[which(sub2016$float_stuck == T)],col=2, pch="*")
    points(sub2016$posixdate[which(sub2016$floods == T)], sub2016$Valor[which(sub2016$floods == T)],col=4, pch="*")
    legend("topleft", c("Range", as.character(round(range(sub2016$Valor, na.rm=T), digits = 1))))}

  jump <- data.frame(2006:2016,rep(NA,11))
  if(length(water$posixdate[which(water$posixdate == "2006-03-26 01:45:00")]) > 0){
    jump[1,2] <- water$posixdate[which(water$posixdate == "2006-03-26 01:45:00")+1] - water$posixdate[which(water$posixdate == "2006-03-26 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2007-03-25 01:45:00")]) > 0){
    jump[2,2] <- water$posixdate[which(water$posixdate == "2007-03-25 01:45:00")+1] - water$posixdate[which(water$posixdate == "2007-03-25 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2008-03-30 01:45:00")]) > 0){
    jump[3,2] <- water$posixdate[which(water$posixdate == "2008-03-30 01:45:00")+1] - water$posixdate[which(water$posixdate == "2008-03-30 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2009-03-29 01:45:00")]) > 0){
    jump[4,2] <- water$posixdate[which(water$posixdate == "2009-03-29 01:45:00")+1] - water$posixdate[which(water$posixdate == "2009-03-29 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2010-03-28 01:45:00")]) > 0){
    jump[5,2] <- water$posixdate[which(water$posixdate == "2010-03-28 01:45:00")+1] - water$posixdate[which(water$posixdate == "2010-03-28 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2011-03-27 01:45:00")]) > 0){
    jump[6,2] <- water$posixdate[which(water$posixdate == "2011-03-27 01:45:00")+1] - water$posixdate[which(water$posixdate == "2011-03-27 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2012-03-25 01:45:00")]) > 0){
    jump[7,2] <- water$posixdate[which(water$posixdate == "2012-03-25 01:45:00")+1] - water$posixdate[which(water$posixdate == "2012-03-25 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2013-03-31 01:45:00")]) > 0){
    jump[8,2] <- water$posixdate[which(water$posixdate == "2013-03-31 01:45:00")+1] - water$posixdate[which(water$posixdate == "2013-03-31 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2014-03-30 01:45:00")]) > 0){
    jump[9,2] <- water$posixdate[which(water$posixdate == "2014-03-30 01:45:00")+1] - water$posixdate[which(water$posixdate == "2014-03-30 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2015-03-29 01:45:00")]) > 0){
    jump[10,2] <- water$posixdate[which(water$posixdate == "2015-03-29 01:45:00")+1] - water$posixdate[which(water$posixdate == "2015-03-29 01:45:00")]}
  if(length(water$posixdate[which(water$posixdate == "2016-03-27 01:45:00")]) > 0){
    jump[11,2] <- water$posixdate[which(water$posixdate == "2016-03-27 01:45:00")+1] - water$posixdate[which(water$posixdate == "2016-03-27 01:45:00")]}
  
  plot(jump, pch=19,main=paste("Winter time to Summer time ",filename,sep = ""), xlab="Year", ylab="Time between 01:45 and next time step")
  
  #Dublicate check
  if(anyDuplicated(water$posixdate)>0){duplicates_file = (paste("Dublicates in ", single_file))
  } else {duplicates_file = paste("No dublicates in ", single_file)}
  
  if(nrow(water) >0){
    plot(water$posixdate, water$Valor, main=paste(filename),ylab="", xlab="", axes=F, type="n")
    legend("center", paste(duplicates_file), bg="grey90")}
  
  #Time difference
  diff_time <- diff(water$posixdate)
  diff_time[which(as.numeric(diff_time) == 15)] <- NA
  plot(water$posixdate[1:(length(water$posixdate)-1)],diff_time, ylim=c(0,max(as.numeric(diff_time), na.rm=T)),pch=19, xlab="Date", ylab="Time difference [min]", main="Time Difference not 15min")
  
  dev.off()
  
}

water_saih(single_file)


### Water level: Cuashi Data####

source("U:/HiWi_ER/CUAHSI/f_write4cuahsi.R")

load("R_Data/A047B17NRIO1.RData") ; data <- water

# Select time frame for CUASHI upload
data <- subset(data, format(posixdate, "%Y") >"2010")

colnames(data) = c("Tag","date", "DataValue", "float_stuck", "floods", "discharge")
n= nrow(data)
data$censorc	<- rep("nc", n)
data$censorc[data$float_stuck] = "lt" # 'less than' for periods float stuck on sediment

write4cuahsi(cdata=data, dest_file=paste("Capella_water", sep=""), sitecode="Capella", varcode="water_stage_15" , methodc="stage_shaft_encoder" , sourcec="SAIH", censorc="", OffsetValue="", qualityc=1, ValueAccuracy="", maxrec=700000)


### Discharge: Cuashi Data####

source("U:/HiWi_ER/CUAHSI/f_write4cuahsi.R")

load("R_Data/A047B17NRIO1.RData") ; data <- water

# Select time frame for CUASHI upload
data <- subset(data, format(posixdate, "%Y") >"2010")

colnames(data) = c("Tag","date", "Valor", "float_stuck", "floods", "DataValue")
n= nrow(data)
data$censorc	<- rep("nc", n)
data$censorc[data$float_stuck] = "lt" # 'less than' for periods float stuck on sediment

write4cuahsi(cdata=data, dest_file=paste("Capella_discha", sep=""), sitecode="Capella", varcode="discharge_15" , methodc="rating_curve" , sourcec="SAIH", censorc="", OffsetValue="", qualityc=1, ValueAccuracy="", maxrec=700000)


### Water level: Final Check####

final <- function(filename){
  
  cuashi_file = paste("CuashiData/", filename, sep="")
  cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )
  
  cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))
  
  sub2011 <- subset(cuashi, format(LocalDateTime, "%Y")=="2011")
  sub2012 <- subset(cuashi, format(LocalDateTime, "%Y")=="2012")
  sub2013 <- subset(cuashi, format(LocalDateTime, "%Y")=="2013")
  sub2014 <- subset(cuashi, format(LocalDateTime, "%Y")=="2014")
  sub2015 <- subset(cuashi, format(LocalDateTime, "%Y")=="2015")
  sub2016 <- subset(cuashi, format(LocalDateTime, "%Y")=="2016")
  #filename <- cuashi$SiteCode[1]
  
  pdf(paste("Plots/",filename,"_Cuashi.pdf",sep = ""),width=10,height=8)
  
  if(nrow(cuashi) >0){
    plot(cuashi$LocalDateTime, cuashi$DataValue, main=paste(filename), ylab = "Water level [m]", xlab= "Time", type="l")
    points(cuashi$LocalDateTime[which(cuashi$CensorCode == "lt")], cuashi$DataValue[which(cuashi$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(cuashi$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$LocalDateTime, sub2011$DataValue, main=paste(filename,"_2011",sep=""), ylab = "Water level [m]", xlab= "Time", type="l")
    points(sub2011$LocalDateTime[which(sub2011$CensorCode == "lt")], sub2011$DataValue[which(sub2011$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2011$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$LocalDateTime, sub2012$DataValue, main=paste(filename,"_2012",sep=""), ylab = "Water level [m]", xlab= "Time", type="l")
    points(sub2012$LocalDateTime[which(sub2012$CensorCode == "lt")], sub2012$DataValue[which(sub2012$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2012$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$LocalDateTime, sub2013$DataValue, main=paste(filename,"_2013",sep=""), ylab = "Water level [m]", xlab= "Time", type="l")
    points(sub2013$LocalDateTime[which(sub2013$CensorCode == "lt")], sub2013$DataValue[which(sub2013$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2013$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$LocalDateTime, sub2014$DataValue, main=paste(filename,"_2014",sep=""), ylab = "Water level [m]", xlab= "Time", type="l")
    points(sub2014$LocalDateTime[which(sub2014$CensorCode == "lt")], sub2014$DataValue[which(sub2014$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2014$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$LocalDateTime, sub2015$DataValue, main=paste(filename,"_2015",sep=""), ylab = "Water level [m]", xlab= "Time", type="l")
    points(sub2015$LocalDateTime[which(sub2015$CensorCode == "lt")], sub2015$DataValue[which(sub2015$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2015$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$LocalDateTime, sub2016$DataValue, main=paste(filename,"_2016",sep=""), ylab = "Water level [m]", xlab= "Time", type="l")
    points(sub2016$LocalDateTime[which(sub2016$CensorCode == "lt")], sub2016$DataValue[which(sub2016$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2016$DataValue, na.rm=T), digits = 1))))}
  
  dev.off()
  
}

dir(path="CuashiData")
final("A047B17NRIO1.csv")


### Discharge: Final Check####

final <- function(filename){
  
  cuashi_file = paste("CuashiData/", filename, sep="")
  cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )
  
  cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))
  
  sub2011 <- subset(cuashi, format(LocalDateTime, "%Y")=="2011")
  sub2012 <- subset(cuashi, format(LocalDateTime, "%Y")=="2012")
  sub2013 <- subset(cuashi, format(LocalDateTime, "%Y")=="2013")
  sub2014 <- subset(cuashi, format(LocalDateTime, "%Y")=="2014")
  sub2015 <- subset(cuashi, format(LocalDateTime, "%Y")=="2015")
  sub2016 <- subset(cuashi, format(LocalDateTime, "%Y")=="2016")
  #filename <- cuashi$SiteCode[1]
  
  pdf(paste("Plots/",filename,"_Cuashi.pdf",sep = ""),width=10,height=8)
  
  if(nrow(cuashi) >0){
    plot(cuashi$LocalDateTime, cuashi$DataValue, main=paste(filename), ylab = "Discharge [m3/s]", xlab= "Time", type="l")
    points(cuashi$LocalDateTime[which(cuashi$CensorCode == "lt")], cuashi$DataValue[which(cuashi$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(cuashi$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$LocalDateTime, sub2011$DataValue, main=paste(filename,"_2011",sep=""), ylab = "Discharge [m3/s]", xlab= "Time", type="l")
    points(sub2011$LocalDateTime[which(sub2011$CensorCode == "lt")], sub2011$DataValue[which(sub2011$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2011$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$LocalDateTime, sub2012$DataValue, main=paste(filename,"_2012",sep=""), ylab = "Discharge [m3/s]", xlab= "Time", type="l")
    points(sub2012$LocalDateTime[which(sub2012$CensorCode == "lt")], sub2012$DataValue[which(sub2012$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2012$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$LocalDateTime, sub2013$DataValue, main=paste(filename,"_2013",sep=""), ylab = "Discharge [m3/s]", xlab= "Time", type="l")
    points(sub2013$LocalDateTime[which(sub2013$CensorCode == "lt")], sub2013$DataValue[which(sub2013$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2013$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$LocalDateTime, sub2014$DataValue, main=paste(filename,"_2014",sep=""), ylab = "Discharge [m3/s]", xlab= "Time", type="l")
    points(sub2014$LocalDateTime[which(sub2014$CensorCode == "lt")], sub2014$DataValue[which(sub2014$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2014$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$LocalDateTime, sub2015$DataValue, main=paste(filename,"_2015",sep=""), ylab = "Discharge [m3/s]", xlab= "Time", type="l")
    points(sub2015$LocalDateTime[which(sub2015$CensorCode == "lt")], sub2015$DataValue[which(sub2015$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2015$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$LocalDateTime, sub2016$DataValue, main=paste(filename,"_2016",sep=""), ylab = "Discharge [m3/s]", xlab= "Time", type="l")
    points(sub2016$LocalDateTime[which(sub2016$CensorCode == "lt")], sub2016$DataValue[which(sub2016$CensorCode == "lt")],col=2, pch=19)
    legend("topleft", c("Range", as.character(round(range(sub2016$DataValue, na.rm=T), digits = 1))))}
  
  dev.off()
  
}

dir(path="CuashiData")
final("A047B17NRIO1_RC.csv")



### Stuff not needed (anymore), maybe useful later ####

# start end of period stuck
start_end = 0*water.dif

wl = length(water.dif)
i = 1

while (i <= wl)
{
  if (float_stuck[i] == F)
  {  
    start_end[i]=0
    i = i+1
    next
  }
  pl = which(water.dif[i:wl] != 0)[1] - 1
  if (is.na(pl)) pl=wl-i+1
  start_end[c(i,(i+pl-1))]=pl
  i = i+ pl
}

start_end[wl+1] = NA
water$start_end = start_end
float <- water[which(water$start_end != 0),]  
float$float_stuck = NULL
float$sed_rem =NULL

save(file = paste0("R_Data/","Float_stuck_",filename,".RData"), list="float") 
write.csv(float, file = paste0("R_Data/","Float_stuck_",filename,".csv"))

sed_rem = water.dif <= -0.2
sed_rem[wl+1] = NA
water$sed_rem = sed_rem



